from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GithubProfile")


@attr.s(auto_attribs=True)
class GithubProfile:
    """A github profile

    Attributes:
        login (Union[Unset, str]): The username of the user
        id (Union[Unset, str]): The id of the user
        node_id (Union[Unset, str]): The node id of the user
        avatar_url (Union[Unset, str]): The avatar url of the user
        gravatar_id (Union[Unset, str]): The gravatar id of the user
        url (Union[Unset, str]): The url of the user
        html_url (Union[Unset, str]): The html url of the user
        followers_url (Union[Unset, str]): The followers url of the user
        following_url (Union[Unset, str]): The following url of the user
        gists_url (Union[Unset, str]): The gists url of the user
        starred_url (Union[Unset, str]): The starred url of the user
        subscriptions_url (Union[Unset, str]): The starred url of the user
        organizations_url (Union[Unset, str]): The organizations url of the user
        repos_url (Union[Unset, str]): The repos url of the user
        events_url (Union[Unset, str]): The events url of the user
        received_events_url (Union[Unset, str]): The received events url of the user
        type (Union[Unset, str]): The type of the user
        site_admin (Union[Unset, bool]): Indicates if the user is a site admin
        name (Union[Unset, str]): The name of the user
        company (Union[Unset, str]): The company of the user
        blog (Union[Unset, str]): The blog of the user
        location (Union[Unset, str]): The location of the user
        email (Union[Unset, str]): The email of the user
        hireable (Union[Unset, str]): Indicates whether the user is hireable
        bio (Union[Unset, str]): The bio of the user
        twitter_username (Union[Unset, str]): The twitter username of the user
        public_repos (Union[Unset, float]): The number of public repos for the user
        public_gists (Union[Unset, float]): The number of public gists for the user
        followers (Union[Unset, float]): The number of followers for the user
        following (Union[Unset, float]): The number of following for the user
        created_at (Union[Unset, str]): Account creation date
        updated_at (Union[Unset, str]): Account update date
        private_gists (Union[Unset, float]): The number of private gists for the user
        total_private_repos (Union[Unset, float]): The number of total repos for the user
        owned_private_repos (Union[Unset, float]): The number of owned private repos for the user
        disk_usage (Union[Unset, float]): The user's disk usage
        collaborators (Union[Unset, float]): The number of collaborators for the user
        two_factor_authentication (Union[Unset, bool]): Indicates whether 2FA is required
    """

    login: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    node_id: Union[Unset, str] = UNSET
    avatar_url: Union[Unset, str] = UNSET
    gravatar_id: Union[Unset, str] = UNSET
    url: Union[Unset, str] = UNSET
    html_url: Union[Unset, str] = UNSET
    followers_url: Union[Unset, str] = UNSET
    following_url: Union[Unset, str] = UNSET
    gists_url: Union[Unset, str] = UNSET
    starred_url: Union[Unset, str] = UNSET
    subscriptions_url: Union[Unset, str] = UNSET
    organizations_url: Union[Unset, str] = UNSET
    repos_url: Union[Unset, str] = UNSET
    events_url: Union[Unset, str] = UNSET
    received_events_url: Union[Unset, str] = UNSET
    type: Union[Unset, str] = UNSET
    site_admin: Union[Unset, bool] = UNSET
    name: Union[Unset, str] = UNSET
    company: Union[Unset, str] = UNSET
    blog: Union[Unset, str] = UNSET
    location: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    hireable: Union[Unset, str] = UNSET
    bio: Union[Unset, str] = UNSET
    twitter_username: Union[Unset, str] = UNSET
    public_repos: Union[Unset, float] = UNSET
    public_gists: Union[Unset, float] = UNSET
    followers: Union[Unset, float] = UNSET
    following: Union[Unset, float] = UNSET
    created_at: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    private_gists: Union[Unset, float] = UNSET
    total_private_repos: Union[Unset, float] = UNSET
    owned_private_repos: Union[Unset, float] = UNSET
    disk_usage: Union[Unset, float] = UNSET
    collaborators: Union[Unset, float] = UNSET
    two_factor_authentication: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        login = self.login
        id = self.id
        node_id = self.node_id
        avatar_url = self.avatar_url
        gravatar_id = self.gravatar_id
        url = self.url
        html_url = self.html_url
        followers_url = self.followers_url
        following_url = self.following_url
        gists_url = self.gists_url
        starred_url = self.starred_url
        subscriptions_url = self.subscriptions_url
        organizations_url = self.organizations_url
        repos_url = self.repos_url
        events_url = self.events_url
        received_events_url = self.received_events_url
        type = self.type
        site_admin = self.site_admin
        name = self.name
        company = self.company
        blog = self.blog
        location = self.location
        email = self.email
        hireable = self.hireable
        bio = self.bio
        twitter_username = self.twitter_username
        public_repos = self.public_repos
        public_gists = self.public_gists
        followers = self.followers
        following = self.following
        created_at = self.created_at
        updated_at = self.updated_at
        private_gists = self.private_gists
        total_private_repos = self.total_private_repos
        owned_private_repos = self.owned_private_repos
        disk_usage = self.disk_usage
        collaborators = self.collaborators
        two_factor_authentication = self.two_factor_authentication

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if login is not UNSET:
            field_dict["login"] = login
        if id is not UNSET:
            field_dict["id"] = id
        if node_id is not UNSET:
            field_dict["node_id"] = node_id
        if avatar_url is not UNSET:
            field_dict["avatar_url"] = avatar_url
        if gravatar_id is not UNSET:
            field_dict["gravatar_id"] = gravatar_id
        if url is not UNSET:
            field_dict["url"] = url
        if html_url is not UNSET:
            field_dict["html_url"] = html_url
        if followers_url is not UNSET:
            field_dict["followers_url"] = followers_url
        if following_url is not UNSET:
            field_dict["following_url"] = following_url
        if gists_url is not UNSET:
            field_dict["gists_url"] = gists_url
        if starred_url is not UNSET:
            field_dict["starred_url"] = starred_url
        if subscriptions_url is not UNSET:
            field_dict["subscriptions_url"] = subscriptions_url
        if organizations_url is not UNSET:
            field_dict["organizations_url"] = organizations_url
        if repos_url is not UNSET:
            field_dict["repos_url"] = repos_url
        if events_url is not UNSET:
            field_dict["events_url"] = events_url
        if received_events_url is not UNSET:
            field_dict["received_events_url"] = received_events_url
        if type is not UNSET:
            field_dict["type"] = type
        if site_admin is not UNSET:
            field_dict["site_admin"] = site_admin
        if name is not UNSET:
            field_dict["name"] = name
        if company is not UNSET:
            field_dict["company"] = company
        if blog is not UNSET:
            field_dict["blog"] = blog
        if location is not UNSET:
            field_dict["location"] = location
        if email is not UNSET:
            field_dict["email"] = email
        if hireable is not UNSET:
            field_dict["hireable"] = hireable
        if bio is not UNSET:
            field_dict["bio"] = bio
        if twitter_username is not UNSET:
            field_dict["twitter_username"] = twitter_username
        if public_repos is not UNSET:
            field_dict["public_repos"] = public_repos
        if public_gists is not UNSET:
            field_dict["public_gists"] = public_gists
        if followers is not UNSET:
            field_dict["followers"] = followers
        if following is not UNSET:
            field_dict["following"] = following
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if private_gists is not UNSET:
            field_dict["private_gists"] = private_gists
        if total_private_repos is not UNSET:
            field_dict["total_private_repos"] = total_private_repos
        if owned_private_repos is not UNSET:
            field_dict["owned_private_repos"] = owned_private_repos
        if disk_usage is not UNSET:
            field_dict["disk_usage"] = disk_usage
        if collaborators is not UNSET:
            field_dict["collaborators"] = collaborators
        if two_factor_authentication is not UNSET:
            field_dict["two_factor_authentication"] = two_factor_authentication

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        login = d.pop("login", UNSET) or UNSET

        id = d.pop("id", UNSET) or UNSET

        node_id = d.pop("node_id", UNSET) or UNSET

        avatar_url = d.pop("avatar_url", UNSET) or UNSET

        gravatar_id = d.pop("gravatar_id", UNSET) or UNSET

        url = d.pop("url", UNSET) or UNSET

        html_url = d.pop("html_url", UNSET) or UNSET

        followers_url = d.pop("followers_url", UNSET) or UNSET

        following_url = d.pop("following_url", UNSET) or UNSET

        gists_url = d.pop("gists_url", UNSET) or UNSET

        starred_url = d.pop("starred_url", UNSET) or UNSET

        subscriptions_url = d.pop("subscriptions_url", UNSET) or UNSET

        organizations_url = d.pop("organizations_url", UNSET) or UNSET

        repos_url = d.pop("repos_url", UNSET) or UNSET

        events_url = d.pop("events_url", UNSET) or UNSET

        received_events_url = d.pop("received_events_url", UNSET) or UNSET

        type = d.pop("type", UNSET) or UNSET

        site_admin = d.pop("site_admin", UNSET) or UNSET

        name = d.pop("name", UNSET) or UNSET

        company = d.pop("company", UNSET) or UNSET

        blog = d.pop("blog", UNSET) or UNSET

        location = d.pop("location", UNSET) or UNSET

        email = d.pop("email", UNSET) or UNSET

        hireable = d.pop("hireable", UNSET) or UNSET

        bio = d.pop("bio", UNSET) or UNSET

        twitter_username = d.pop("twitter_username", UNSET) or UNSET

        public_repos = d.pop("public_repos", UNSET) or UNSET

        public_gists = d.pop("public_gists", UNSET) or UNSET

        followers = d.pop("followers", UNSET) or UNSET

        following = d.pop("following", UNSET) or UNSET

        created_at = d.pop("created_at", UNSET) or UNSET

        updated_at = d.pop("updated_at", UNSET) or UNSET

        private_gists = d.pop("private_gists", UNSET) or UNSET

        total_private_repos = d.pop("total_private_repos", UNSET) or UNSET

        owned_private_repos = d.pop("owned_private_repos", UNSET) or UNSET

        disk_usage = d.pop("disk_usage", UNSET) or UNSET

        collaborators = d.pop("collaborators", UNSET) or UNSET

        two_factor_authentication = d.pop("two_factor_authentication", UNSET) or UNSET

        github_profile = cls(
            login=login,
            id=id,
            node_id=node_id,
            avatar_url=avatar_url,
            gravatar_id=gravatar_id,
            url=url,
            html_url=html_url,
            followers_url=followers_url,
            following_url=following_url,
            gists_url=gists_url,
            starred_url=starred_url,
            subscriptions_url=subscriptions_url,
            organizations_url=organizations_url,
            repos_url=repos_url,
            events_url=events_url,
            received_events_url=received_events_url,
            type=type,
            site_admin=site_admin,
            name=name,
            company=company,
            blog=blog,
            location=location,
            email=email,
            hireable=hireable,
            bio=bio,
            twitter_username=twitter_username,
            public_repos=public_repos,
            public_gists=public_gists,
            followers=followers,
            following=following,
            created_at=created_at,
            updated_at=updated_at,
            private_gists=private_gists,
            total_private_repos=total_private_repos,
            owned_private_repos=owned_private_repos,
            disk_usage=disk_usage,
            collaborators=collaborators,
            two_factor_authentication=two_factor_authentication,
        )

        github_profile.additional_properties = d
        return github_profile

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_event_reminders_overrides_item import GoogleEventRemindersOverridesItem


T = TypeVar("T", bound="GoogleEventReminders")


@attr.s(auto_attribs=True)
class GoogleEventReminders:
    """The reminders of the event

    Attributes:
        use_default (Union[Unset, bool]): Whether the reminders use default
        overrides (Union[Unset, List['GoogleEventRemindersOverridesItem']]): The overrides of the reminders
    """

    use_default: Union[Unset, bool] = UNSET
    overrides: Union[Unset, List["GoogleEventRemindersOverridesItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        use_default = self.use_default
        overrides: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.overrides, Unset):
            overrides = []
            for overrides_item_data in self.overrides:
                overrides_item = overrides_item_data.to_dict()

                overrides.append(overrides_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if use_default is not UNSET:
            field_dict["useDefault"] = use_default
        if overrides is not UNSET:
            field_dict["overrides"] = overrides

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_event_reminders_overrides_item import GoogleEventRemindersOverridesItem

        d = src_dict.copy()
        use_default = d.pop("useDefault", UNSET) or UNSET

        overrides = []
        _overrides = d.pop("overrides", UNSET) or UNSET
        for overrides_item_data in _overrides or []:
            overrides_item = GoogleEventRemindersOverridesItem.from_dict(overrides_item_data)

            overrides.append(overrides_item)

        google_event_reminders = cls(
            use_default=use_default,
            overrides=overrides,
        )

        google_event_reminders.additional_properties = d
        return google_event_reminders

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class AccountStatus(str, Enum):
    ACTIVE = "active"
    INACTIVE = "inactive"
    PENDING = "pending"
    REMOVED = "removed"

    def __str__(self) -> str:
        return str(self.value)



# TODO figure out how the generator handles multiple methods (GET, POST, etc.) with the same name

import json
from langchain.base_language import BaseLanguageModel
from langchain.callbacks.manager import AsyncCallbackManagerForToolRun, CallbackManagerForToolRun
from langchain.output_parsers import PydanticOutputParser
from langchain.prompts.prompt import PromptTemplate
from langchain.tools import BaseTool
from pydantic import BaseModel, Field
from typing import Callable, List, Optional
from urllib.parse import quote

from ..client import AuthenticatedClient
from ..client.api.accounts import find_all_accounts_for_user
from ..client.api.auth import get_user_auth_token
from ..client.models import Auth, ReturnedAccountDto
from ..client.types import Response

from ..client.api.sources import github_controller_repos
from ..client.models import GithubRepo
from ..client.api.sources import github_controller_profile
from ..client.models import GithubProfile


class GithubAuthenticatorTool(BaseTool):
    name = 'github_authenticator_tool'
    description = """Useful for when a user's Github account is not authenticated.

    Input to the tool should be an empty string.

    The response from the tool will be a URL that you return to the user for them to complete auth.
    The URL will be your final answer.
    """

    client: AuthenticatedClient
    user_id: str
    server: str = 'https://ui.bruinen.co'
    source_policy_id: str = None
    redirect_url: str

    def _run(
        self,
        query: str,
        run_manager: Optional[CallbackManagerForToolRun] = None
    ) -> str:
        """Run the tool."""
        response: Response[Auth] = get_user_auth_token.sync_detailed(client=self.client, user_id=self.user_id)
        user_token = response.parsed.access_token

        if self.source_policy_id is not None:
            source = [{ 'name': 'github', 'sourcePolicyId': self.source_policy_id }]
        else:
            source = [{ 'name': 'github' }]
        encoded_source = quote(json.dumps(source))
        
        return self.server + '/connect' + '?userToken=' + quote(user_token) + '&sources=' + encoded_source + '&defaultRedirectUrl=' + quote(self.redirect_url)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GithubGetReposTool(BaseTool):
    name = "github_get_repos_tool"
    description = """Useful for when you need to get a user's Github repos.
    
    Input should be the question that you want to know the answer to.
    
    Output will be the text response from the Github API.
    """

    client: AuthenticatedClient
    user_id: str
    parse_output: Optional[Callable[[List["GithubRepo"]], str]] = None
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Github account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "github":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Github account; you should try authenticating Github first."
        else:
            response: Response[List["GithubRepo"]] = github_controller_repos.sync_detailed(
                client=self.client,
                account_id=account_id
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Github repos."

            if self.parse_output is None:
                # Call each response item's to_dict() method and return the result as a JSON string
                return json.dumps(list(map(lambda x: x.to_dict(), response.parsed)))
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)


class GithubGetProfileTool(BaseTool):
    name = "github_get_profile_tool"
    description = """Useful for when you need to get a user's Github profile.
    
    Input should be the question that you want to know the answer to.
    
    Output will be the text response from the Github API.
    """

    client: AuthenticatedClient
    user_id: str
    parse_output: Optional[Callable[[GithubProfile], str]] = None
    
    def _run(self, query: str, run_manager: Optional[CallbackManagerForToolRun] = None) -> str:
        """Run the tool."""
        
        response: Response[List["ReturnedAccountDto"]] = find_all_accounts_for_user.sync_detailed(
            client=self.client, user_id=self.user_id
        )
        if not 200 <= response.status_code < 300:
            return "Error pulling the user's Github account."
        accounts: List["ReturnedAccountDto"] = response.parsed

        account_id = ""
        for account in accounts:
            if account.source == "github":
                account_id = account.id
        if account_id == "":
            return "The user has not connected their Github account; you should try authenticating Github first."
        else:
            response: Response[GithubProfile] = github_controller_profile.sync_detailed(
                client=self.client,
                account_id=account_id
            )
            if not 200 <= response.status_code < 300:
                return "Error when attempting to get the user's Github profile."

            if self.parse_output is None:
                return json.dumps(response.parsed.to_dict())
            else:
                return self.parse_output(response.parsed, query)

    # TODO implement async version
    async def _arun(
        self,
        query: str,
        run_manager: Optional[AsyncCallbackManagerForToolRun] = None,
    ) -> str:
        """Run the tool asynchronously."""
        return await self._run(query, run_manager)




from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleDraftMessagePayloadBody")


@attr.s(auto_attribs=True)
class GoogleDraftMessagePayloadBody:
    """The body of the payload

    Attributes:
        size (Union[Unset, float]):
        data (Union[Unset, str]):
        attachment_id (Union[Unset, str]):
    """

    size: Union[Unset, float] = UNSET
    data: Union[Unset, str] = UNSET
    attachment_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        size = self.size
        data = self.data
        attachment_id = self.attachment_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if size is not UNSET:
            field_dict["size"] = size
        if data is not UNSET:
            field_dict["data"] = data
        if attachment_id is not UNSET:
            field_dict["attachmentId"] = attachment_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        size = d.pop("size", UNSET) or UNSET

        data = d.pop("data", UNSET) or UNSET

        attachment_id = d.pop("attachmentId", UNSET) or UNSET

        google_draft_message_payload_body = cls(
            size=size,
            data=data,
            attachment_id=attachment_id,
        )

        google_draft_message_payload_body.additional_properties = d
        return google_draft_message_payload_body

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_event_conference_data_conference_solution_key import (
        GoogleEventConferenceDataConferenceSolutionKey,
    )


T = TypeVar("T", bound="GoogleEventConferenceDataConferenceSolution")


@attr.s(auto_attribs=True)
class GoogleEventConferenceDataConferenceSolution:
    """The conferenceSolution of the conferenceData

    Attributes:
        key (Union[Unset, GoogleEventConferenceDataConferenceSolutionKey]): The key of the conferenceSolution
        name (Union[Unset, str]): The name of the conferenceSolution
        icon_uri (Union[Unset, str]): The iconUri of the conferenceSolution
    """

    key: Union[Unset, "GoogleEventConferenceDataConferenceSolutionKey"] = UNSET
    name: Union[Unset, str] = UNSET
    icon_uri: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        key: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.key, Unset):
            key = self.key.to_dict()

        name = self.name
        icon_uri = self.icon_uri

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if key is not UNSET:
            field_dict["key"] = key
        if name is not UNSET:
            field_dict["name"] = name
        if icon_uri is not UNSET:
            field_dict["iconUri"] = icon_uri

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_event_conference_data_conference_solution_key import (
            GoogleEventConferenceDataConferenceSolutionKey,
        )

        d = src_dict.copy()
        _key = d.pop("key", UNSET) or UNSET
        key: Union[Unset, GoogleEventConferenceDataConferenceSolutionKey]
        if isinstance(_key, Unset):
            key = UNSET
        else:
            key = GoogleEventConferenceDataConferenceSolutionKey.from_dict(_key)

        name = d.pop("name", UNSET) or UNSET

        icon_uri = d.pop("iconUri", UNSET) or UNSET

        google_event_conference_data_conference_solution = cls(
            key=key,
            name=name,
            icon_uri=icon_uri,
        )

        google_event_conference_data_conference_solution.additional_properties = d
        return google_event_conference_data_conference_solution

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from enum import Enum


class ConfirmationStatus(str, Enum):
    CONFIRMED = "confirmed"
    DELIVERED = "delivered"
    DENIED = "denied"
    ERROR = "error"
    PENDING = "pending"

    def __str__(self) -> str:
        return str(self.value)



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GithubRepoPermissions")


@attr.s(auto_attribs=True)
class GithubRepoPermissions:
    """The permissions object for the repo

    Attributes:
        admin (Union[Unset, bool]): Whether the repo has admin permissions
        maintain (Union[Unset, bool]): Whether the repo has maintain permissions
        push (Union[Unset, bool]): Whether the repo has push permissions
        triage (Union[Unset, bool]): Whether the repo has triage permissions
        pull (Union[Unset, bool]): Whether the repo has pull permissions
    """

    admin: Union[Unset, bool] = UNSET
    maintain: Union[Unset, bool] = UNSET
    push: Union[Unset, bool] = UNSET
    triage: Union[Unset, bool] = UNSET
    pull: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        admin = self.admin
        maintain = self.maintain
        push = self.push
        triage = self.triage
        pull = self.pull

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if admin is not UNSET:
            field_dict["admin"] = admin
        if maintain is not UNSET:
            field_dict["maintain"] = maintain
        if push is not UNSET:
            field_dict["push"] = push
        if triage is not UNSET:
            field_dict["triage"] = triage
        if pull is not UNSET:
            field_dict["pull"] = pull

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        admin = d.pop("admin", UNSET) or UNSET

        maintain = d.pop("maintain", UNSET) or UNSET

        push = d.pop("push", UNSET) or UNSET

        triage = d.pop("triage", UNSET) or UNSET

        pull = d.pop("pull", UNSET) or UNSET

        github_repo_permissions = cls(
            admin=admin,
            maintain=maintain,
            push=push,
            triage=triage,
            pull=pull,
        )

        github_repo_permissions.additional_properties = d
        return github_repo_permissions

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from datetime import datetime
from typing import ForwardRef, Union, get_args, get_origin

from langchain import SQLDatabase, SQLDatabaseChain
from langchain.base_language import BaseLanguageModel
from langchain.prompts.prompt import PromptTemplate
from sqlalchemy import JSON, Boolean, Column, DateTime, Float, Integer, MetaData, String, Table, create_engine
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker
from sqlalchemy.schema import CreateTable

from ...client.types import Unset


class SQLParser:
    def __init__(self, llm: BaseLanguageModel):
        self.llm = llm

    def parse(self, input, query):
        # Check if the input is a list (the SQL parser is only available for resources that return lists)
        if not (isinstance(input, list) or get_origin(input) is list):
            raise TypeError("SQL parser is only available for resources that return lists")

        input_class = type(input[0])
        input_class_name = input_class.__name__

        class_attrs = {
            "__tablename__": input_class_name.lower(),
            "temporary_id": Column(Integer, primary_key=True),
        }

        # Could maybe use SQLAlchemy or another library like SQLAlchemy Utils for this mapping
        def map_sql_types(type_var):
            if type_var is str:
                return String
            elif type_var is int:
                return Integer
            elif type_var is datetime:
                return DateTime
            elif type_var is float:
                return Float
            elif type_var is dict:
                return JSON
            elif type_var is bool:
                return Boolean
            elif get_origin(type_var) is dict:
                return JSON
            elif get_origin(type_var) is list:
                return JSON
            elif type(type_var) == ForwardRef:
                return JSON
            else:
                raise ValueError(f"Unknown type when mapping Typing to SQLAlchemy: {type_var}")

        # TODO check if there's a better way to pull these
        for field_name, field_type in input_class.__dict__["__annotations__"].items():
            if get_origin(field_type) is Union:
                # First argument should be Unset, otherwise raise an error
                if get_args(field_type)[0] is not Unset:
                    raise ValueError("Union type with first value not Unset not supported")
                class_attrs[field_name] = Column(map_sql_types(get_args(field_type)[1]), nullable=True)
            else:
                class_attrs[field_name] = Column(map_sql_types(field_type))

        engine = create_engine(f"sqlite:///{input_class_name.lower()}.db", echo=True)
        Base = declarative_base()
        sql_class = type(input_class_name, (Base,), class_attrs)
        Base.metadata.create_all(engine)
        Session = sessionmaker(bind=engine)
        session = Session()

        data = []
        for line in input:
            data.append(line.to_dict())
        session.bulk_insert_mappings(sql_class, data)
        session.commit()
        session.close()

        metadata = MetaData()
        metadata.reflect(bind=engine)
        sql_table = Table(input_class_name.lower(), metadata, autoload=True, autoload_with=engine)

        template_table = "\nOnly use the following tables:\n" + str(CreateTable(sql_table).compile(engine))
        # If the table isn't too large, would be better to also include example data
        template_start = """Given an input question, first create a syntactically correct {dialect} query to run, then look at the results of the query and return the answer. Use the following format:

        Question: "Question here"
        SQLQuery: "SQL Query to run"
        SQLResult: "Result of the SQLQuery"
        Answer: "Final answer here"
        """
        template_end = "Question: {input}"

        prompt_template = template_start + template_table + template_end

        prompt = PromptTemplate(input_variables=["input", "dialect"], template=prompt_template)
        db = SQLDatabase.from_uri(f"sqlite:///{input_class_name.lower()}.db")
        db_chain = SQLDatabaseChain.from_llm(llm=self.llm, db=db, prompt=prompt, verbose=True)
        res = db_chain.run(query)

        # Drop the created table so it doesn't overwrite later
        sql_table.drop(engine)

        return res



""" Contains all the data models used in inputs/outputs """

from .account_status import AccountStatus
from .accounts_controller_deactivate_response_200 import AccountsControllerDeactivateResponse200
from .accounts_controller_get_accounts_response_200_item import AccountsControllerGetAccountsResponse200Item
from .auth import Auth
from .client_usage_data import ClientUsageData
from .confirmation_status import ConfirmationStatus
from .connection_requests_controller_find_all_response_200_item import (
    ConnectionRequestsControllerFindAllResponse200Item,
)
from .create_confirm_dto import CreateConfirmDto
from .create_confirm_dto_params import CreateConfirmDtoParams
from .create_confirm_returned_dto import CreateConfirmReturnedDto
from .create_connection_request_dto import CreateConnectionRequestDto
from .create_connection_request_dto_source import CreateConnectionRequestDtoSource
from .create_user_dto import CreateUserDto
from .credential_provider import CredentialProvider
from .endpoint_data import EndpointData
from .get_response_200 import GetResponse200
from .github_profile import GithubProfile
from .github_repo import GithubRepo
from .github_repo_owner import GithubRepoOwner
from .github_repo_permissions import GithubRepoPermissions
from .google_calendar import GoogleCalendar
from .google_calendar_conference_properties import GoogleCalendarConferenceProperties
from .google_calendars import GoogleCalendars
from .google_calendars_calendars_item import GoogleCalendarsCalendarsItem
from .google_calendars_calendars_item_conference_properties import GoogleCalendarsCalendarsItemConferenceProperties
from .google_calendars_calendars_item_default_reminders import GoogleCalendarsCalendarsItemDefaultReminders
from .google_calendars_calendars_item_notification_settings import GoogleCalendarsCalendarsItemNotificationSettings
from .google_calendars_calendars_item_notification_settings_notifications_item import (
    GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem,
)
from .google_draft import GoogleDraft
from .google_draft_message import GoogleDraftMessage
from .google_draft_message_payload import GoogleDraftMessagePayload
from .google_draft_message_payload_body import GoogleDraftMessagePayloadBody
from .google_draft_message_payload_headers_item import GoogleDraftMessagePayloadHeadersItem
from .google_draft_message_payload_parts_item import GoogleDraftMessagePayloadPartsItem
from .google_draft_message_payload_parts_item_body import GoogleDraftMessagePayloadPartsItemBody
from .google_draft_message_payload_parts_item_headers_item import GoogleDraftMessagePayloadPartsItemHeadersItem
from .google_draft_message_payload_parts_item_parts_item import GoogleDraftMessagePayloadPartsItemPartsItem
from .google_draft_message_payload_parts_item_parts_item_body import GoogleDraftMessagePayloadPartsItemPartsItemBody
from .google_draft_message_payload_parts_item_parts_item_headers_item import (
    GoogleDraftMessagePayloadPartsItemPartsItemHeadersItem,
)
from .google_drafts import GoogleDrafts
from .google_drafts_drafts_item import GoogleDraftsDraftsItem
from .google_drafts_drafts_item_message import GoogleDraftsDraftsItemMessage
from .google_event import GoogleEvent
from .google_event_attachments_item import GoogleEventAttachmentsItem
from .google_event_attendees_item import GoogleEventAttendeesItem
from .google_event_conference_data import GoogleEventConferenceData
from .google_event_conference_data_conference_solution import GoogleEventConferenceDataConferenceSolution
from .google_event_conference_data_conference_solution_key import GoogleEventConferenceDataConferenceSolutionKey
from .google_event_conference_data_create_request import GoogleEventConferenceDataCreateRequest
from .google_event_conference_data_create_request_conference_solution_key import (
    GoogleEventConferenceDataCreateRequestConferenceSolutionKey,
)
from .google_event_conference_data_create_request_status import GoogleEventConferenceDataCreateRequestStatus
from .google_event_conference_data_entry_points_item import GoogleEventConferenceDataEntryPointsItem
from .google_event_creator import GoogleEventCreator
from .google_event_end import GoogleEventEnd
from .google_event_extended_properties import GoogleEventExtendedProperties
from .google_event_extended_properties_private import GoogleEventExtendedPropertiesPrivate
from .google_event_extended_properties_shared import GoogleEventExtendedPropertiesShared
from .google_event_gadget import GoogleEventGadget
from .google_event_gadget_preferences import GoogleEventGadgetPreferences
from .google_event_organizer import GoogleEventOrganizer
from .google_event_original_start_time import GoogleEventOriginalStartTime
from .google_event_reminders import GoogleEventReminders
from .google_event_reminders_overrides_item import GoogleEventRemindersOverridesItem
from .google_event_source import GoogleEventSource
from .google_event_start import GoogleEventStart
from .google_events import GoogleEvents
from .google_events_default_reminders_item import GoogleEventsDefaultRemindersItem
from .google_events_events_item import GoogleEventsEventsItem
from .google_events_events_item_attachments_item import GoogleEventsEventsItemAttachmentsItem
from .google_events_events_item_attendees_item import GoogleEventsEventsItemAttendeesItem
from .google_events_events_item_conference_data import GoogleEventsEventsItemConferenceData
from .google_events_events_item_conference_data_conference_solution import (
    GoogleEventsEventsItemConferenceDataConferenceSolution,
)
from .google_events_events_item_conference_data_conference_solution_key import (
    GoogleEventsEventsItemConferenceDataConferenceSolutionKey,
)
from .google_events_events_item_conference_data_create_request import GoogleEventsEventsItemConferenceDataCreateRequest
from .google_events_events_item_conference_data_create_request_conference_solution_key import (
    GoogleEventsEventsItemConferenceDataCreateRequestConferenceSolutionKey,
)
from .google_events_events_item_conference_data_create_request_status import (
    GoogleEventsEventsItemConferenceDataCreateRequestStatus,
)
from .google_events_events_item_conference_data_entry_points_item import (
    GoogleEventsEventsItemConferenceDataEntryPointsItem,
)
from .google_events_events_item_creator import GoogleEventsEventsItemCreator
from .google_events_events_item_end import GoogleEventsEventsItemEnd
from .google_events_events_item_extended_properties import GoogleEventsEventsItemExtendedProperties
from .google_events_events_item_extended_properties_private import GoogleEventsEventsItemExtendedPropertiesPrivate
from .google_events_events_item_extended_properties_shared import GoogleEventsEventsItemExtendedPropertiesShared
from .google_events_events_item_gadget import GoogleEventsEventsItemGadget
from .google_events_events_item_gadget_preferences import GoogleEventsEventsItemGadgetPreferences
from .google_events_events_item_organizer import GoogleEventsEventsItemOrganizer
from .google_events_events_item_original_start_time import GoogleEventsEventsItemOriginalStartTime
from .google_events_events_item_reminders import GoogleEventsEventsItemReminders
from .google_events_events_item_reminders_overrides_item import GoogleEventsEventsItemRemindersOverridesItem
from .google_events_events_item_source import GoogleEventsEventsItemSource
from .google_events_events_item_start import GoogleEventsEventsItemStart
from .google_label import GoogleLabel
from .google_label_color import GoogleLabelColor
from .google_labels import GoogleLabels
from .google_labels_labels_item import GoogleLabelsLabelsItem
from .google_message import GoogleMessage
from .google_message_payload import GoogleMessagePayload
from .google_message_payload_body import GoogleMessagePayloadBody
from .google_message_payload_headers_item import GoogleMessagePayloadHeadersItem
from .google_message_payload_parts_item import GoogleMessagePayloadPartsItem
from .google_message_payload_parts_item_body import GoogleMessagePayloadPartsItemBody
from .google_message_payload_parts_item_headers_item import GoogleMessagePayloadPartsItemHeadersItem
from .google_messages import GoogleMessages
from .google_messages_messages_item import GoogleMessagesMessagesItem
from .google_parsed_draft import GoogleParsedDraft
from .google_parsed_draft_attachments_item import GoogleParsedDraftAttachmentsItem
from .google_parsed_draft_headers import GoogleParsedDraftHeaders
from .google_parsed_draft_headers_bcc_item import GoogleParsedDraftHeadersBccItem
from .google_parsed_draft_headers_cc_item import GoogleParsedDraftHeadersCcItem
from .google_parsed_draft_headers_from import GoogleParsedDraftHeadersFrom
from .google_parsed_draft_headers_to_item import GoogleParsedDraftHeadersToItem
from .google_parsed_drafts import GoogleParsedDrafts
from .google_parsed_drafts_drafts_item import GoogleParsedDraftsDraftsItem
from .google_parsed_drafts_drafts_item_attachments_item import GoogleParsedDraftsDraftsItemAttachmentsItem
from .google_parsed_drafts_drafts_item_headers import GoogleParsedDraftsDraftsItemHeaders
from .google_parsed_drafts_drafts_item_headers_bcc_item import GoogleParsedDraftsDraftsItemHeadersBccItem
from .google_parsed_drafts_drafts_item_headers_cc_item import GoogleParsedDraftsDraftsItemHeadersCcItem
from .google_parsed_drafts_drafts_item_headers_from import GoogleParsedDraftsDraftsItemHeadersFrom
from .google_parsed_drafts_drafts_item_headers_to_item import GoogleParsedDraftsDraftsItemHeadersToItem
from .google_parsed_message import GoogleParsedMessage
from .google_parsed_message_attachments_item import GoogleParsedMessageAttachmentsItem
from .google_parsed_message_headers import GoogleParsedMessageHeaders
from .google_parsed_message_headers_bcc_item import GoogleParsedMessageHeadersBccItem
from .google_parsed_message_headers_cc_item import GoogleParsedMessageHeadersCcItem
from .google_parsed_message_headers_from import GoogleParsedMessageHeadersFrom
from .google_parsed_message_headers_to_item import GoogleParsedMessageHeadersToItem
from .google_parsed_messages import GoogleParsedMessages
from .google_parsed_messages_messages_item import GoogleParsedMessagesMessagesItem
from .google_parsed_messages_messages_item_attachments_item import GoogleParsedMessagesMessagesItemAttachmentsItem
from .google_parsed_messages_messages_item_headers import GoogleParsedMessagesMessagesItemHeaders
from .google_parsed_messages_messages_item_headers_bcc_item import GoogleParsedMessagesMessagesItemHeadersBccItem
from .google_parsed_messages_messages_item_headers_cc_item import GoogleParsedMessagesMessagesItemHeadersCcItem
from .google_parsed_messages_messages_item_headers_from import GoogleParsedMessagesMessagesItemHeadersFrom
from .google_parsed_messages_messages_item_headers_to_item import GoogleParsedMessagesMessagesItemHeadersToItem
from .google_parsed_thread import GoogleParsedThread
from .google_parsed_thread_messages_item import GoogleParsedThreadMessagesItem
from .google_parsed_thread_messages_item_attachments_item import GoogleParsedThreadMessagesItemAttachmentsItem
from .google_parsed_thread_messages_item_headers import GoogleParsedThreadMessagesItemHeaders
from .google_parsed_thread_messages_item_headers_bcc_item import GoogleParsedThreadMessagesItemHeadersBccItem
from .google_parsed_thread_messages_item_headers_cc_item import GoogleParsedThreadMessagesItemHeadersCcItem
from .google_parsed_thread_messages_item_headers_from import GoogleParsedThreadMessagesItemHeadersFrom
from .google_parsed_thread_messages_item_headers_to_item import GoogleParsedThreadMessagesItemHeadersToItem
from .google_parsed_threads import GoogleParsedThreads
from .google_parsed_threads_threads_item import GoogleParsedThreadsThreadsItem
from .google_parsed_threads_threads_item_messages_item import GoogleParsedThreadsThreadsItemMessagesItem
from .google_parsed_threads_threads_item_messages_item_attachments_item import (
    GoogleParsedThreadsThreadsItemMessagesItemAttachmentsItem,
)
from .google_parsed_threads_threads_item_messages_item_headers import GoogleParsedThreadsThreadsItemMessagesItemHeaders
from .google_parsed_threads_threads_item_messages_item_headers_bcc_item import (
    GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem,
)
from .google_parsed_threads_threads_item_messages_item_headers_cc_item import (
    GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem,
)
from .google_parsed_threads_threads_item_messages_item_headers_from import (
    GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom,
)
from .google_parsed_threads_threads_item_messages_item_headers_to_item import (
    GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem,
)
from .google_profile import GoogleProfile
from .google_thread import GoogleThread
from .google_thread_messages_item import GoogleThreadMessagesItem
from .google_thread_messages_item_payload import GoogleThreadMessagesItemPayload
from .google_thread_messages_item_payload_body import GoogleThreadMessagesItemPayloadBody
from .google_thread_messages_item_payload_headers_item import GoogleThreadMessagesItemPayloadHeadersItem
from .google_thread_messages_item_payload_parts_item import GoogleThreadMessagesItemPayloadPartsItem
from .google_thread_messages_item_payload_parts_item_body import GoogleThreadMessagesItemPayloadPartsItemBody
from .google_thread_messages_item_payload_parts_item_headers_item import (
    GoogleThreadMessagesItemPayloadPartsItemHeadersItem,
)
from .google_threads import GoogleThreads
from .google_threads_threads_item import GoogleThreadsThreadsItem
from .returned_account_dto import ReturnedAccountDto
from .returned_confirm_dto import ReturnedConfirmDto
from .returned_confirm_dto_params import ReturnedConfirmDtoParams
from .returned_connection_request_dto import ReturnedConnectionRequestDto
from .returned_connection_request_dto_source import ReturnedConnectionRequestDtoSource
from .returned_source_policy_dto import ReturnedSourcePolicyDto
from .returned_user_dto import ReturnedUserDto
from .source_data import SourceData
from .source_policy_status import SourcePolicyStatus
from .source_type import SourceType
from .sources_controller_get_metadata_for_source_response_200 import SourcesControllerGetMetadataForSourceResponse200
from .update_user_dto import UpdateUserDto
from .upsert_user_dto import UpsertUserDto
from .usage_controller_find_all_response_200_item import UsageControllerFindAllResponse200Item

__all__ = (
    "AccountsControllerDeactivateResponse200",
    "AccountsControllerGetAccountsResponse200Item",
    "AccountStatus",
    "Auth",
    "ClientUsageData",
    "ConfirmationStatus",
    "ConnectionRequestsControllerFindAllResponse200Item",
    "CreateConfirmDto",
    "CreateConfirmDtoParams",
    "CreateConfirmReturnedDto",
    "CreateConnectionRequestDto",
    "CreateConnectionRequestDtoSource",
    "CreateUserDto",
    "CredentialProvider",
    "EndpointData",
    "GetResponse200",
    "GithubProfile",
    "GithubRepo",
    "GithubRepoOwner",
    "GithubRepoPermissions",
    "GoogleCalendar",
    "GoogleCalendarConferenceProperties",
    "GoogleCalendars",
    "GoogleCalendarsCalendarsItem",
    "GoogleCalendarsCalendarsItemConferenceProperties",
    "GoogleCalendarsCalendarsItemDefaultReminders",
    "GoogleCalendarsCalendarsItemNotificationSettings",
    "GoogleCalendarsCalendarsItemNotificationSettingsNotificationsItem",
    "GoogleDraft",
    "GoogleDraftMessage",
    "GoogleDraftMessagePayload",
    "GoogleDraftMessagePayloadBody",
    "GoogleDraftMessagePayloadHeadersItem",
    "GoogleDraftMessagePayloadPartsItem",
    "GoogleDraftMessagePayloadPartsItemBody",
    "GoogleDraftMessagePayloadPartsItemHeadersItem",
    "GoogleDraftMessagePayloadPartsItemPartsItem",
    "GoogleDraftMessagePayloadPartsItemPartsItemBody",
    "GoogleDraftMessagePayloadPartsItemPartsItemHeadersItem",
    "GoogleDrafts",
    "GoogleDraftsDraftsItem",
    "GoogleDraftsDraftsItemMessage",
    "GoogleEvent",
    "GoogleEventAttachmentsItem",
    "GoogleEventAttendeesItem",
    "GoogleEventConferenceData",
    "GoogleEventConferenceDataConferenceSolution",
    "GoogleEventConferenceDataConferenceSolutionKey",
    "GoogleEventConferenceDataCreateRequest",
    "GoogleEventConferenceDataCreateRequestConferenceSolutionKey",
    "GoogleEventConferenceDataCreateRequestStatus",
    "GoogleEventConferenceDataEntryPointsItem",
    "GoogleEventCreator",
    "GoogleEventEnd",
    "GoogleEventExtendedProperties",
    "GoogleEventExtendedPropertiesPrivate",
    "GoogleEventExtendedPropertiesShared",
    "GoogleEventGadget",
    "GoogleEventGadgetPreferences",
    "GoogleEventOrganizer",
    "GoogleEventOriginalStartTime",
    "GoogleEventReminders",
    "GoogleEventRemindersOverridesItem",
    "GoogleEvents",
    "GoogleEventsDefaultRemindersItem",
    "GoogleEventsEventsItem",
    "GoogleEventsEventsItemAttachmentsItem",
    "GoogleEventsEventsItemAttendeesItem",
    "GoogleEventsEventsItemConferenceData",
    "GoogleEventsEventsItemConferenceDataConferenceSolution",
    "GoogleEventsEventsItemConferenceDataConferenceSolutionKey",
    "GoogleEventsEventsItemConferenceDataCreateRequest",
    "GoogleEventsEventsItemConferenceDataCreateRequestConferenceSolutionKey",
    "GoogleEventsEventsItemConferenceDataCreateRequestStatus",
    "GoogleEventsEventsItemConferenceDataEntryPointsItem",
    "GoogleEventsEventsItemCreator",
    "GoogleEventsEventsItemEnd",
    "GoogleEventsEventsItemExtendedProperties",
    "GoogleEventsEventsItemExtendedPropertiesPrivate",
    "GoogleEventsEventsItemExtendedPropertiesShared",
    "GoogleEventsEventsItemGadget",
    "GoogleEventsEventsItemGadgetPreferences",
    "GoogleEventsEventsItemOrganizer",
    "GoogleEventsEventsItemOriginalStartTime",
    "GoogleEventsEventsItemReminders",
    "GoogleEventsEventsItemRemindersOverridesItem",
    "GoogleEventsEventsItemSource",
    "GoogleEventsEventsItemStart",
    "GoogleEventSource",
    "GoogleEventStart",
    "GoogleLabel",
    "GoogleLabelColor",
    "GoogleLabels",
    "GoogleLabelsLabelsItem",
    "GoogleMessage",
    "GoogleMessagePayload",
    "GoogleMessagePayloadBody",
    "GoogleMessagePayloadHeadersItem",
    "GoogleMessagePayloadPartsItem",
    "GoogleMessagePayloadPartsItemBody",
    "GoogleMessagePayloadPartsItemHeadersItem",
    "GoogleMessages",
    "GoogleMessagesMessagesItem",
    "GoogleParsedDraft",
    "GoogleParsedDraftAttachmentsItem",
    "GoogleParsedDraftHeaders",
    "GoogleParsedDraftHeadersBccItem",
    "GoogleParsedDraftHeadersCcItem",
    "GoogleParsedDraftHeadersFrom",
    "GoogleParsedDraftHeadersToItem",
    "GoogleParsedDrafts",
    "GoogleParsedDraftsDraftsItem",
    "GoogleParsedDraftsDraftsItemAttachmentsItem",
    "GoogleParsedDraftsDraftsItemHeaders",
    "GoogleParsedDraftsDraftsItemHeadersBccItem",
    "GoogleParsedDraftsDraftsItemHeadersCcItem",
    "GoogleParsedDraftsDraftsItemHeadersFrom",
    "GoogleParsedDraftsDraftsItemHeadersToItem",
    "GoogleParsedMessage",
    "GoogleParsedMessageAttachmentsItem",
    "GoogleParsedMessageHeaders",
    "GoogleParsedMessageHeadersBccItem",
    "GoogleParsedMessageHeadersCcItem",
    "GoogleParsedMessageHeadersFrom",
    "GoogleParsedMessageHeadersToItem",
    "GoogleParsedMessages",
    "GoogleParsedMessagesMessagesItem",
    "GoogleParsedMessagesMessagesItemAttachmentsItem",
    "GoogleParsedMessagesMessagesItemHeaders",
    "GoogleParsedMessagesMessagesItemHeadersBccItem",
    "GoogleParsedMessagesMessagesItemHeadersCcItem",
    "GoogleParsedMessagesMessagesItemHeadersFrom",
    "GoogleParsedMessagesMessagesItemHeadersToItem",
    "GoogleParsedThread",
    "GoogleParsedThreadMessagesItem",
    "GoogleParsedThreadMessagesItemAttachmentsItem",
    "GoogleParsedThreadMessagesItemHeaders",
    "GoogleParsedThreadMessagesItemHeadersBccItem",
    "GoogleParsedThreadMessagesItemHeadersCcItem",
    "GoogleParsedThreadMessagesItemHeadersFrom",
    "GoogleParsedThreadMessagesItemHeadersToItem",
    "GoogleParsedThreads",
    "GoogleParsedThreadsThreadsItem",
    "GoogleParsedThreadsThreadsItemMessagesItem",
    "GoogleParsedThreadsThreadsItemMessagesItemAttachmentsItem",
    "GoogleParsedThreadsThreadsItemMessagesItemHeaders",
    "GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem",
    "GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem",
    "GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom",
    "GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem",
    "GoogleProfile",
    "GoogleThread",
    "GoogleThreadMessagesItem",
    "GoogleThreadMessagesItemPayload",
    "GoogleThreadMessagesItemPayloadBody",
    "GoogleThreadMessagesItemPayloadHeadersItem",
    "GoogleThreadMessagesItemPayloadPartsItem",
    "GoogleThreadMessagesItemPayloadPartsItemBody",
    "GoogleThreadMessagesItemPayloadPartsItemHeadersItem",
    "GoogleThreads",
    "GoogleThreadsThreadsItem",
    "ReturnedAccountDto",
    "ReturnedConfirmDto",
    "ReturnedConfirmDtoParams",
    "ReturnedConnectionRequestDto",
    "ReturnedConnectionRequestDtoSource",
    "ReturnedSourcePolicyDto",
    "ReturnedUserDto",
    "SourceData",
    "SourcePolicyStatus",
    "SourcesControllerGetMetadataForSourceResponse200",
    "SourceType",
    "UpdateUserDto",
    "UpsertUserDto",
    "UsageControllerFindAllResponse200Item",
)



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_labels_labels_item import GoogleLabelsLabelsItem


T = TypeVar("T", bound="GoogleLabels")


@attr.s(auto_attribs=True)
class GoogleLabels:
    """Your google labels

    Attributes:
        result_size_estimate (Union[Unset, float]): The result size estimate for your labels
        labels (Union[Unset, List['GoogleLabelsLabelsItem']]): A list of your google labels
    """

    result_size_estimate: Union[Unset, float] = UNSET
    labels: Union[Unset, List["GoogleLabelsLabelsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        result_size_estimate = self.result_size_estimate
        labels: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.labels, Unset):
            labels = []
            for labels_item_data in self.labels:
                labels_item = labels_item_data.to_dict()

                labels.append(labels_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if result_size_estimate is not UNSET:
            field_dict["resultSizeEstimate"] = result_size_estimate
        if labels is not UNSET:
            field_dict["labels"] = labels

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_labels_labels_item import GoogleLabelsLabelsItem

        d = src_dict.copy()
        result_size_estimate = d.pop("resultSizeEstimate", UNSET) or UNSET

        labels = []
        _labels = d.pop("labels", UNSET) or UNSET
        for labels_item_data in _labels or []:
            labels_item = GoogleLabelsLabelsItem.from_dict(labels_item_data)

            labels.append(labels_item)

        google_labels = cls(
            result_size_estimate=result_size_estimate,
            labels=labels,
        )

        google_labels.additional_properties = d
        return google_labels

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar

import attr

T = TypeVar("T", bound="EndpointData")


@attr.s(auto_attribs=True)
class EndpointData:
    """
    Attributes:
        endpoint (str):
        total_requests (float):
        total_successes (float):
        total_failures (float):
    """

    endpoint: str
    total_requests: float
    total_successes: float
    total_failures: float
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        endpoint = self.endpoint
        total_requests = self.total_requests
        total_successes = self.total_successes
        total_failures = self.total_failures

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "endpoint": endpoint,
                "totalRequests": total_requests,
                "totalSuccesses": total_successes,
                "totalFailures": total_failures,
            }
        )

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        endpoint = d.pop("endpoint")

        total_requests = d.pop("totalRequests")

        total_successes = d.pop("totalSuccesses")

        total_failures = d.pop("totalFailures")

        endpoint_data = cls(
            endpoint=endpoint,
            total_requests=total_requests,
            total_successes=total_successes,
            total_failures=total_failures,
        )

        endpoint_data.additional_properties = d
        return endpoint_data

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_event_attachments_item import GoogleEventAttachmentsItem
    from ..models.google_event_attendees_item import GoogleEventAttendeesItem
    from ..models.google_event_conference_data import GoogleEventConferenceData
    from ..models.google_event_creator import GoogleEventCreator
    from ..models.google_event_end import GoogleEventEnd
    from ..models.google_event_extended_properties import GoogleEventExtendedProperties
    from ..models.google_event_gadget import GoogleEventGadget
    from ..models.google_event_organizer import GoogleEventOrganizer
    from ..models.google_event_original_start_time import GoogleEventOriginalStartTime
    from ..models.google_event_reminders import GoogleEventReminders
    from ..models.google_event_source import GoogleEventSource
    from ..models.google_event_start import GoogleEventStart


T = TypeVar("T", bound="GoogleEvent")


@attr.s(auto_attribs=True)
class GoogleEvent:
    """Your google calendar's event

    Attributes:
        kind (Union[Unset, str]): The kind of the event
        etag (Union[Unset, str]): The etag of the event
        id (Union[Unset, str]): The id of the event
        status (Union[Unset, str]): The status of the event
        html_link (Union[Unset, str]): The htmlLink of the event
        created (Union[Unset, str]): The created date of the event
        updated (Union[Unset, str]): The updated date of the event
        summary (Union[Unset, str]): The summary of the event
        description (Union[Unset, str]): The description of the event
        location (Union[Unset, str]): The location of the event
        color_id (Union[Unset, str]): The colorId of the event
        creator (Union[Unset, GoogleEventCreator]): The creator of the event
        organizer (Union[Unset, GoogleEventOrganizer]): The organizer of the event
        start (Union[Unset, GoogleEventStart]): The start of the event
        end (Union[Unset, GoogleEventEnd]): The end of the event
        end_time_unspecified (Union[Unset, bool]): Whether the endTime is unspecified
        recurrence (Union[Unset, List[str]]): The recurrence of the event
        recurring_event_id (Union[Unset, str]): The recurringEventId of the event
        original_start_time (Union[Unset, GoogleEventOriginalStartTime]): The originalStartTime of the event
        transparency (Union[Unset, str]): The transparency of the event
        visibility (Union[Unset, str]): The visibility of the event
        i_cal_uid (Union[Unset, str]): The iCalUID of the event
        sequence (Union[Unset, float]): The sequence of the event
        attendees (Union[Unset, List['GoogleEventAttendeesItem']]): The attendees of the event
        attendees_omitted (Union[Unset, bool]): Whether attendees are omitted
        extended_properties (Union[Unset, GoogleEventExtendedProperties]): The extendedProperties of the event
        hangout_link (Union[Unset, str]): The hangoutLink of the event
        conference_data (Union[Unset, GoogleEventConferenceData]): The conferenceData of the event
        gadget (Union[Unset, GoogleEventGadget]): The gadget of the event
        anyone_can_add_self (Union[Unset, bool]): Whether anyone can add self
        guests_can_invite_others (Union[Unset, bool]): Whether guests can invite others
        guests_can_modify (Union[Unset, bool]): Whether guests can modify
        guests_can_see_other_guests (Union[Unset, bool]): Whether guests can see other guests
        private_copy (Union[Unset, bool]): Whether the event is a private copy
        locked (Union[Unset, bool]): Whether the event is locked
        reminders (Union[Unset, GoogleEventReminders]): The reminders of the event
        source (Union[Unset, GoogleEventSource]): The source of the event
        attachments (Union[Unset, List['GoogleEventAttachmentsItem']]): The attachments of the event
        event_type (Union[Unset, str]): The eventType of the event
    """

    kind: Union[Unset, str] = UNSET
    etag: Union[Unset, str] = UNSET
    id: Union[Unset, str] = UNSET
    status: Union[Unset, str] = UNSET
    html_link: Union[Unset, str] = UNSET
    created: Union[Unset, str] = UNSET
    updated: Union[Unset, str] = UNSET
    summary: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    location: Union[Unset, str] = UNSET
    color_id: Union[Unset, str] = UNSET
    creator: Union[Unset, "GoogleEventCreator"] = UNSET
    organizer: Union[Unset, "GoogleEventOrganizer"] = UNSET
    start: Union[Unset, "GoogleEventStart"] = UNSET
    end: Union[Unset, "GoogleEventEnd"] = UNSET
    end_time_unspecified: Union[Unset, bool] = UNSET
    recurrence: Union[Unset, List[str]] = UNSET
    recurring_event_id: Union[Unset, str] = UNSET
    original_start_time: Union[Unset, "GoogleEventOriginalStartTime"] = UNSET
    transparency: Union[Unset, str] = UNSET
    visibility: Union[Unset, str] = UNSET
    i_cal_uid: Union[Unset, str] = UNSET
    sequence: Union[Unset, float] = UNSET
    attendees: Union[Unset, List["GoogleEventAttendeesItem"]] = UNSET
    attendees_omitted: Union[Unset, bool] = UNSET
    extended_properties: Union[Unset, "GoogleEventExtendedProperties"] = UNSET
    hangout_link: Union[Unset, str] = UNSET
    conference_data: Union[Unset, "GoogleEventConferenceData"] = UNSET
    gadget: Union[Unset, "GoogleEventGadget"] = UNSET
    anyone_can_add_self: Union[Unset, bool] = UNSET
    guests_can_invite_others: Union[Unset, bool] = UNSET
    guests_can_modify: Union[Unset, bool] = UNSET
    guests_can_see_other_guests: Union[Unset, bool] = UNSET
    private_copy: Union[Unset, bool] = UNSET
    locked: Union[Unset, bool] = UNSET
    reminders: Union[Unset, "GoogleEventReminders"] = UNSET
    source: Union[Unset, "GoogleEventSource"] = UNSET
    attachments: Union[Unset, List["GoogleEventAttachmentsItem"]] = UNSET
    event_type: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind
        etag = self.etag
        id = self.id
        status = self.status
        html_link = self.html_link
        created = self.created
        updated = self.updated
        summary = self.summary
        description = self.description
        location = self.location
        color_id = self.color_id
        creator: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.creator, Unset):
            creator = self.creator.to_dict()

        organizer: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.organizer, Unset):
            organizer = self.organizer.to_dict()

        start: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.start, Unset):
            start = self.start.to_dict()

        end: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.end, Unset):
            end = self.end.to_dict()

        end_time_unspecified = self.end_time_unspecified
        recurrence: Union[Unset, List[str]] = UNSET
        if not isinstance(self.recurrence, Unset):
            recurrence = self.recurrence

        recurring_event_id = self.recurring_event_id
        original_start_time: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.original_start_time, Unset):
            original_start_time = self.original_start_time.to_dict()

        transparency = self.transparency
        visibility = self.visibility
        i_cal_uid = self.i_cal_uid
        sequence = self.sequence
        attendees: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attendees, Unset):
            attendees = []
            for attendees_item_data in self.attendees:
                attendees_item = attendees_item_data.to_dict()

                attendees.append(attendees_item)

        attendees_omitted = self.attendees_omitted
        extended_properties: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.extended_properties, Unset):
            extended_properties = self.extended_properties.to_dict()

        hangout_link = self.hangout_link
        conference_data: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.conference_data, Unset):
            conference_data = self.conference_data.to_dict()

        gadget: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.gadget, Unset):
            gadget = self.gadget.to_dict()

        anyone_can_add_self = self.anyone_can_add_self
        guests_can_invite_others = self.guests_can_invite_others
        guests_can_modify = self.guests_can_modify
        guests_can_see_other_guests = self.guests_can_see_other_guests
        private_copy = self.private_copy
        locked = self.locked
        reminders: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.reminders, Unset):
            reminders = self.reminders.to_dict()

        source: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.source, Unset):
            source = self.source.to_dict()

        attachments: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.attachments, Unset):
            attachments = []
            for attachments_item_data in self.attachments:
                attachments_item = attachments_item_data.to_dict()

                attachments.append(attachments_item)

        event_type = self.event_type

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if etag is not UNSET:
            field_dict["etag"] = etag
        if id is not UNSET:
            field_dict["id"] = id
        if status is not UNSET:
            field_dict["status"] = status
        if html_link is not UNSET:
            field_dict["htmlLink"] = html_link
        if created is not UNSET:
            field_dict["created"] = created
        if updated is not UNSET:
            field_dict["updated"] = updated
        if summary is not UNSET:
            field_dict["summary"] = summary
        if description is not UNSET:
            field_dict["description"] = description
        if location is not UNSET:
            field_dict["location"] = location
        if color_id is not UNSET:
            field_dict["colorId"] = color_id
        if creator is not UNSET:
            field_dict["creator"] = creator
        if organizer is not UNSET:
            field_dict["organizer"] = organizer
        if start is not UNSET:
            field_dict["start"] = start
        if end is not UNSET:
            field_dict["end"] = end
        if end_time_unspecified is not UNSET:
            field_dict["endTimeUnspecified"] = end_time_unspecified
        if recurrence is not UNSET:
            field_dict["recurrence"] = recurrence
        if recurring_event_id is not UNSET:
            field_dict["recurringEventId"] = recurring_event_id
        if original_start_time is not UNSET:
            field_dict["originalStartTime"] = original_start_time
        if transparency is not UNSET:
            field_dict["transparency"] = transparency
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if i_cal_uid is not UNSET:
            field_dict["iCalUID"] = i_cal_uid
        if sequence is not UNSET:
            field_dict["sequence"] = sequence
        if attendees is not UNSET:
            field_dict["attendees"] = attendees
        if attendees_omitted is not UNSET:
            field_dict["attendeesOmitted"] = attendees_omitted
        if extended_properties is not UNSET:
            field_dict["extendedProperties"] = extended_properties
        if hangout_link is not UNSET:
            field_dict["hangoutLink"] = hangout_link
        if conference_data is not UNSET:
            field_dict["conferenceData"] = conference_data
        if gadget is not UNSET:
            field_dict["gadget"] = gadget
        if anyone_can_add_self is not UNSET:
            field_dict["anyoneCanAddSelf"] = anyone_can_add_self
        if guests_can_invite_others is not UNSET:
            field_dict["guestsCanInviteOthers"] = guests_can_invite_others
        if guests_can_modify is not UNSET:
            field_dict["guestsCanModify"] = guests_can_modify
        if guests_can_see_other_guests is not UNSET:
            field_dict["guestsCanSeeOtherGuests"] = guests_can_see_other_guests
        if private_copy is not UNSET:
            field_dict["privateCopy"] = private_copy
        if locked is not UNSET:
            field_dict["locked"] = locked
        if reminders is not UNSET:
            field_dict["reminders"] = reminders
        if source is not UNSET:
            field_dict["source"] = source
        if attachments is not UNSET:
            field_dict["attachments"] = attachments
        if event_type is not UNSET:
            field_dict["eventType"] = event_type

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_event_attachments_item import GoogleEventAttachmentsItem
        from ..models.google_event_attendees_item import GoogleEventAttendeesItem
        from ..models.google_event_conference_data import GoogleEventConferenceData
        from ..models.google_event_creator import GoogleEventCreator
        from ..models.google_event_end import GoogleEventEnd
        from ..models.google_event_extended_properties import GoogleEventExtendedProperties
        from ..models.google_event_gadget import GoogleEventGadget
        from ..models.google_event_organizer import GoogleEventOrganizer
        from ..models.google_event_original_start_time import GoogleEventOriginalStartTime
        from ..models.google_event_reminders import GoogleEventReminders
        from ..models.google_event_source import GoogleEventSource
        from ..models.google_event_start import GoogleEventStart

        d = src_dict.copy()
        kind = d.pop("kind", UNSET) or UNSET

        etag = d.pop("etag", UNSET) or UNSET

        id = d.pop("id", UNSET) or UNSET

        status = d.pop("status", UNSET) or UNSET

        html_link = d.pop("htmlLink", UNSET) or UNSET

        created = d.pop("created", UNSET) or UNSET

        updated = d.pop("updated", UNSET) or UNSET

        summary = d.pop("summary", UNSET) or UNSET

        description = d.pop("description", UNSET) or UNSET

        location = d.pop("location", UNSET) or UNSET

        color_id = d.pop("colorId", UNSET) or UNSET

        _creator = d.pop("creator", UNSET) or UNSET
        creator: Union[Unset, GoogleEventCreator]
        if isinstance(_creator, Unset):
            creator = UNSET
        else:
            creator = GoogleEventCreator.from_dict(_creator)

        _organizer = d.pop("organizer", UNSET) or UNSET
        organizer: Union[Unset, GoogleEventOrganizer]
        if isinstance(_organizer, Unset):
            organizer = UNSET
        else:
            organizer = GoogleEventOrganizer.from_dict(_organizer)

        _start = d.pop("start", UNSET) or UNSET
        start: Union[Unset, GoogleEventStart]
        if isinstance(_start, Unset):
            start = UNSET
        else:
            start = GoogleEventStart.from_dict(_start)

        _end = d.pop("end", UNSET) or UNSET
        end: Union[Unset, GoogleEventEnd]
        if isinstance(_end, Unset):
            end = UNSET
        else:
            end = GoogleEventEnd.from_dict(_end)

        end_time_unspecified = d.pop("endTimeUnspecified", UNSET) or UNSET

        recurrence = cast(List[str], d.pop("recurrence", UNSET) or UNSET)

        recurring_event_id = d.pop("recurringEventId", UNSET) or UNSET

        _original_start_time = d.pop("originalStartTime", UNSET) or UNSET
        original_start_time: Union[Unset, GoogleEventOriginalStartTime]
        if isinstance(_original_start_time, Unset):
            original_start_time = UNSET
        else:
            original_start_time = GoogleEventOriginalStartTime.from_dict(_original_start_time)

        transparency = d.pop("transparency", UNSET) or UNSET

        visibility = d.pop("visibility", UNSET) or UNSET

        i_cal_uid = d.pop("iCalUID", UNSET) or UNSET

        sequence = d.pop("sequence", UNSET) or UNSET

        attendees = []
        _attendees = d.pop("attendees", UNSET) or UNSET
        for attendees_item_data in _attendees or []:
            attendees_item = GoogleEventAttendeesItem.from_dict(attendees_item_data)

            attendees.append(attendees_item)

        attendees_omitted = d.pop("attendeesOmitted", UNSET) or UNSET

        _extended_properties = d.pop("extendedProperties", UNSET) or UNSET
        extended_properties: Union[Unset, GoogleEventExtendedProperties]
        if isinstance(_extended_properties, Unset):
            extended_properties = UNSET
        else:
            extended_properties = GoogleEventExtendedProperties.from_dict(_extended_properties)

        hangout_link = d.pop("hangoutLink", UNSET) or UNSET

        _conference_data = d.pop("conferenceData", UNSET) or UNSET
        conference_data: Union[Unset, GoogleEventConferenceData]
        if isinstance(_conference_data, Unset):
            conference_data = UNSET
        else:
            conference_data = GoogleEventConferenceData.from_dict(_conference_data)

        _gadget = d.pop("gadget", UNSET) or UNSET
        gadget: Union[Unset, GoogleEventGadget]
        if isinstance(_gadget, Unset):
            gadget = UNSET
        else:
            gadget = GoogleEventGadget.from_dict(_gadget)

        anyone_can_add_self = d.pop("anyoneCanAddSelf", UNSET) or UNSET

        guests_can_invite_others = d.pop("guestsCanInviteOthers", UNSET) or UNSET

        guests_can_modify = d.pop("guestsCanModify", UNSET) or UNSET

        guests_can_see_other_guests = d.pop("guestsCanSeeOtherGuests", UNSET) or UNSET

        private_copy = d.pop("privateCopy", UNSET) or UNSET

        locked = d.pop("locked", UNSET) or UNSET

        _reminders = d.pop("reminders", UNSET) or UNSET
        reminders: Union[Unset, GoogleEventReminders]
        if isinstance(_reminders, Unset):
            reminders = UNSET
        else:
            reminders = GoogleEventReminders.from_dict(_reminders)

        _source = d.pop("source", UNSET) or UNSET
        source: Union[Unset, GoogleEventSource]
        if isinstance(_source, Unset):
            source = UNSET
        else:
            source = GoogleEventSource.from_dict(_source)

        attachments = []
        _attachments = d.pop("attachments", UNSET) or UNSET
        for attachments_item_data in _attachments or []:
            attachments_item = GoogleEventAttachmentsItem.from_dict(attachments_item_data)

            attachments.append(attachments_item)

        event_type = d.pop("eventType", UNSET) or UNSET

        google_event = cls(
            kind=kind,
            etag=etag,
            id=id,
            status=status,
            html_link=html_link,
            created=created,
            updated=updated,
            summary=summary,
            description=description,
            location=location,
            color_id=color_id,
            creator=creator,
            organizer=organizer,
            start=start,
            end=end,
            end_time_unspecified=end_time_unspecified,
            recurrence=recurrence,
            recurring_event_id=recurring_event_id,
            original_start_time=original_start_time,
            transparency=transparency,
            visibility=visibility,
            i_cal_uid=i_cal_uid,
            sequence=sequence,
            attendees=attendees,
            attendees_omitted=attendees_omitted,
            extended_properties=extended_properties,
            hangout_link=hangout_link,
            conference_data=conference_data,
            gadget=gadget,
            anyone_can_add_self=anyone_can_add_self,
            guests_can_invite_others=guests_can_invite_others,
            guests_can_modify=guests_can_modify,
            guests_can_see_other_guests=guests_can_see_other_guests,
            private_copy=private_copy,
            locked=locked,
            reminders=reminders,
            source=source,
            attachments=attachments,
            event_type=event_type,
        )

        google_event.additional_properties = d
        return google_event

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_calendars_calendars_item import GoogleCalendarsCalendarsItem


T = TypeVar("T", bound="GoogleCalendars")


@attr.s(auto_attribs=True)
class GoogleCalendars:
    """Your google calendars

    Attributes:
        kind (Union[Unset, str]): The kind of the calendars
        etag (Union[Unset, str]): The etag of the calendars
        next_page_token (Union[Unset, str]): The next page token
        next_sync_token (Union[Unset, str]): The next sync token
        calendars (Union[Unset, List['GoogleCalendarsCalendarsItem']]): A list of your google calendars
    """

    kind: Union[Unset, str] = UNSET
    etag: Union[Unset, str] = UNSET
    next_page_token: Union[Unset, str] = UNSET
    next_sync_token: Union[Unset, str] = UNSET
    calendars: Union[Unset, List["GoogleCalendarsCalendarsItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        kind = self.kind
        etag = self.etag
        next_page_token = self.next_page_token
        next_sync_token = self.next_sync_token
        calendars: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.calendars, Unset):
            calendars = []
            for calendars_item_data in self.calendars:
                calendars_item = calendars_item_data.to_dict()

                calendars.append(calendars_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if kind is not UNSET:
            field_dict["kind"] = kind
        if etag is not UNSET:
            field_dict["etag"] = etag
        if next_page_token is not UNSET:
            field_dict["nextPageToken"] = next_page_token
        if next_sync_token is not UNSET:
            field_dict["nextSyncToken"] = next_sync_token
        if calendars is not UNSET:
            field_dict["calendars"] = calendars

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_calendars_calendars_item import GoogleCalendarsCalendarsItem

        d = src_dict.copy()
        kind = d.pop("kind", UNSET) or UNSET

        etag = d.pop("etag", UNSET) or UNSET

        next_page_token = d.pop("nextPageToken", UNSET) or UNSET

        next_sync_token = d.pop("nextSyncToken", UNSET) or UNSET

        calendars = []
        _calendars = d.pop("calendars", UNSET) or UNSET
        for calendars_item_data in _calendars or []:
            calendars_item = GoogleCalendarsCalendarsItem.from_dict(calendars_item_data)

            calendars.append(calendars_item)

        google_calendars = cls(
            kind=kind,
            etag=etag,
            next_page_token=next_page_token,
            next_sync_token=next_sync_token,
            calendars=calendars,
        )

        google_calendars.additional_properties = d
        return google_calendars

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="CreateConfirmReturnedDto")


@attr.s(auto_attribs=True)
class CreateConfirmReturnedDto:
    """
    Attributes:
        delivered (bool):
        confirmation_id (str):
        magic_link (Union[Unset, str]):
    """

    delivered: bool
    confirmation_id: str
    magic_link: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        delivered = self.delivered
        confirmation_id = self.confirmation_id
        magic_link = self.magic_link

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "delivered": delivered,
                "confirmationId": confirmation_id,
            }
        )
        if magic_link is not UNSET:
            field_dict["magicLink"] = magic_link

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        delivered = d.pop("delivered")

        confirmation_id = d.pop("confirmationId")

        magic_link = d.pop("magicLink", UNSET) or UNSET

        create_confirm_returned_dto = cls(
            delivered=delivered,
            confirmation_id=confirmation_id,
            magic_link=magic_link,
        )

        create_confirm_returned_dto.additional_properties = d
        return create_confirm_returned_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.github_repo_owner import GithubRepoOwner
    from ..models.github_repo_permissions import GithubRepoPermissions


T = TypeVar("T", bound="GithubRepo")


@attr.s(auto_attribs=True)
class GithubRepo:
    """A github repo

    Attributes:
        id (Union[Unset, str]): The id of the repo
        node_id (Union[Unset, str]): The node id of the repo
        name (Union[Unset, str]): The name of the repo
        full_name (Union[Unset, str]): The full name of the repo
        private (Union[Unset, bool]): Whether the repo is private
        owner (Union[Unset, GithubRepoOwner]): The owner of this github repo
        html_url (Union[Unset, str]): The html url of the repo
        description (Union[Unset, str]): The description of the repo
        fork (Union[Unset, bool]): Whether the repo is a fork
        url (Union[Unset, str]): The url of the repo
        forks_url (Union[Unset, str]): The forks url of the repo
        keys_url (Union[Unset, str]): The keys url of the repo
        collaborators_url (Union[Unset, str]): The collaborators url of the repo
        teams_url (Union[Unset, str]): The teams url of the repo
        hooks_url (Union[Unset, str]): The hooks url of the repo
        issue_events_url (Union[Unset, str]): The issue events url of the repo
        events_url (Union[Unset, str]): The events url of the repo
        assignees_url (Union[Unset, str]): The assignees url of the repo
        branches_url (Union[Unset, str]): The branches url of the repo
        tags_url (Union[Unset, str]): The tags url of the repo
        blobs_url (Union[Unset, str]): The blobs url of the repo
        git_tags_url (Union[Unset, str]): The git tags url of the repo
        git_refs_url (Union[Unset, str]): The git refs url of the repo
        trees_url (Union[Unset, str]): The trees url of the repo
        statuses_url (Union[Unset, str]): The statuses url of the repo
        languages_url (Union[Unset, str]): The languages url of the repo
        stargazers_url (Union[Unset, str]): The stargazers url of the repo
        contributors_url (Union[Unset, str]): The contributors url of the repo
        subscribers_url (Union[Unset, str]): The subscribers url of the repo
        subscription_url (Union[Unset, str]): The subscription url of the repo
        commits_url (Union[Unset, str]): The commits url of the repo
        git_commits_url (Union[Unset, str]): The git commits url of the repo
        comments_url (Union[Unset, str]): The comments url of the repo
        issue_comment_url (Union[Unset, str]): The issue comment url of the repo
        contents_url (Union[Unset, str]): The contents url of the repo
        compare_url (Union[Unset, str]): The compare url of the repo
        merges_url (Union[Unset, str]): The merges url of the repo
        archive_url (Union[Unset, str]): The archive url of the repo
        downloads_url (Union[Unset, str]): The downloads url of the repo
        issues_url (Union[Unset, str]): The issues url of the repo
        pulls_url (Union[Unset, str]): The pulls url of the repo
        milestones_url (Union[Unset, str]): The milestones url of the repo
        notifications_url (Union[Unset, str]): The notifications url of the repo
        labels_url (Union[Unset, str]): The labels url of the repo
        releases_url (Union[Unset, str]): The releases url of the repo
        deployments_url (Union[Unset, str]): The deployments url of the repo
        created_at (Union[Unset, str]): The creation date of the repo
        updated_at (Union[Unset, str]): The update date of the repo
        pushed_at (Union[Unset, str]): The pushed at date of the repo
        git_url (Union[Unset, str]): The git url of the repo
        ssh_url (Union[Unset, str]): The ssh url of the repo
        clone_url (Union[Unset, str]): The clone url of the repo
        svn_url (Union[Unset, str]): The svn url of the repo
        homepage (Union[Unset, str]): The homepage of the repo
        size (Union[Unset, float]): The size of the repo
        stargazers_count (Union[Unset, float]): The stargazers count of the repo
        watchers_count (Union[Unset, float]): The watchers count of the repo
        language (Union[Unset, str]): The language of the repo
        has_issues (Union[Unset, bool]): Whether the repo has issues
        has_projects (Union[Unset, bool]): Whether the repo has projects
        has_downloads (Union[Unset, bool]): Whether the repo has downloads
        has_wiki (Union[Unset, bool]): Whether the repo has a wiki
        has_pages (Union[Unset, bool]): Whether the repo has pages
        forks_count (Union[Unset, float]): The forks count of the repo
        mirror_url (Union[Unset, str]): The mirror url of the repo
        archived (Union[Unset, bool]): Whether the repo is archived
        disabled (Union[Unset, bool]): Whether the repo is disabled
        open_issues_count (Union[Unset, float]): The open issues count of the repo
        license_ (Union[Unset, str]): The license of the repo
        allow_forking (Union[Unset, bool]): Whether the repo allows forking
        is_template (Union[Unset, bool]): Whether the repo is a template
        web_commit_signoff_required (Union[Unset, bool]): Whether the repo requires web commit signoff
        topics (Union[Unset, List[str]]): The topics of the repo
        visibility (Union[Unset, str]): The visibility of the repo
        default_branch (Union[Unset, str]): The default branch of the repo
        permissions (Union[Unset, GithubRepoPermissions]): The permissions object for the repo
    """

    id: Union[Unset, str] = UNSET
    node_id: Union[Unset, str] = UNSET
    name: Union[Unset, str] = UNSET
    full_name: Union[Unset, str] = UNSET
    private: Union[Unset, bool] = UNSET
    owner: Union[Unset, "GithubRepoOwner"] = UNSET
    html_url: Union[Unset, str] = UNSET
    description: Union[Unset, str] = UNSET
    fork: Union[Unset, bool] = UNSET
    url: Union[Unset, str] = UNSET
    forks_url: Union[Unset, str] = UNSET
    keys_url: Union[Unset, str] = UNSET
    collaborators_url: Union[Unset, str] = UNSET
    teams_url: Union[Unset, str] = UNSET
    hooks_url: Union[Unset, str] = UNSET
    issue_events_url: Union[Unset, str] = UNSET
    events_url: Union[Unset, str] = UNSET
    assignees_url: Union[Unset, str] = UNSET
    branches_url: Union[Unset, str] = UNSET
    tags_url: Union[Unset, str] = UNSET
    blobs_url: Union[Unset, str] = UNSET
    git_tags_url: Union[Unset, str] = UNSET
    git_refs_url: Union[Unset, str] = UNSET
    trees_url: Union[Unset, str] = UNSET
    statuses_url: Union[Unset, str] = UNSET
    languages_url: Union[Unset, str] = UNSET
    stargazers_url: Union[Unset, str] = UNSET
    contributors_url: Union[Unset, str] = UNSET
    subscribers_url: Union[Unset, str] = UNSET
    subscription_url: Union[Unset, str] = UNSET
    commits_url: Union[Unset, str] = UNSET
    git_commits_url: Union[Unset, str] = UNSET
    comments_url: Union[Unset, str] = UNSET
    issue_comment_url: Union[Unset, str] = UNSET
    contents_url: Union[Unset, str] = UNSET
    compare_url: Union[Unset, str] = UNSET
    merges_url: Union[Unset, str] = UNSET
    archive_url: Union[Unset, str] = UNSET
    downloads_url: Union[Unset, str] = UNSET
    issues_url: Union[Unset, str] = UNSET
    pulls_url: Union[Unset, str] = UNSET
    milestones_url: Union[Unset, str] = UNSET
    notifications_url: Union[Unset, str] = UNSET
    labels_url: Union[Unset, str] = UNSET
    releases_url: Union[Unset, str] = UNSET
    deployments_url: Union[Unset, str] = UNSET
    created_at: Union[Unset, str] = UNSET
    updated_at: Union[Unset, str] = UNSET
    pushed_at: Union[Unset, str] = UNSET
    git_url: Union[Unset, str] = UNSET
    ssh_url: Union[Unset, str] = UNSET
    clone_url: Union[Unset, str] = UNSET
    svn_url: Union[Unset, str] = UNSET
    homepage: Union[Unset, str] = UNSET
    size: Union[Unset, float] = UNSET
    stargazers_count: Union[Unset, float] = UNSET
    watchers_count: Union[Unset, float] = UNSET
    language: Union[Unset, str] = UNSET
    has_issues: Union[Unset, bool] = UNSET
    has_projects: Union[Unset, bool] = UNSET
    has_downloads: Union[Unset, bool] = UNSET
    has_wiki: Union[Unset, bool] = UNSET
    has_pages: Union[Unset, bool] = UNSET
    forks_count: Union[Unset, float] = UNSET
    mirror_url: Union[Unset, str] = UNSET
    archived: Union[Unset, bool] = UNSET
    disabled: Union[Unset, bool] = UNSET
    open_issues_count: Union[Unset, float] = UNSET
    license_: Union[Unset, str] = UNSET
    allow_forking: Union[Unset, bool] = UNSET
    is_template: Union[Unset, bool] = UNSET
    web_commit_signoff_required: Union[Unset, bool] = UNSET
    topics: Union[Unset, List[str]] = UNSET
    visibility: Union[Unset, str] = UNSET
    default_branch: Union[Unset, str] = UNSET
    permissions: Union[Unset, "GithubRepoPermissions"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        node_id = self.node_id
        name = self.name
        full_name = self.full_name
        private = self.private
        owner: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.owner, Unset):
            owner = self.owner.to_dict()

        html_url = self.html_url
        description = self.description
        fork = self.fork
        url = self.url
        forks_url = self.forks_url
        keys_url = self.keys_url
        collaborators_url = self.collaborators_url
        teams_url = self.teams_url
        hooks_url = self.hooks_url
        issue_events_url = self.issue_events_url
        events_url = self.events_url
        assignees_url = self.assignees_url
        branches_url = self.branches_url
        tags_url = self.tags_url
        blobs_url = self.blobs_url
        git_tags_url = self.git_tags_url
        git_refs_url = self.git_refs_url
        trees_url = self.trees_url
        statuses_url = self.statuses_url
        languages_url = self.languages_url
        stargazers_url = self.stargazers_url
        contributors_url = self.contributors_url
        subscribers_url = self.subscribers_url
        subscription_url = self.subscription_url
        commits_url = self.commits_url
        git_commits_url = self.git_commits_url
        comments_url = self.comments_url
        issue_comment_url = self.issue_comment_url
        contents_url = self.contents_url
        compare_url = self.compare_url
        merges_url = self.merges_url
        archive_url = self.archive_url
        downloads_url = self.downloads_url
        issues_url = self.issues_url
        pulls_url = self.pulls_url
        milestones_url = self.milestones_url
        notifications_url = self.notifications_url
        labels_url = self.labels_url
        releases_url = self.releases_url
        deployments_url = self.deployments_url
        created_at = self.created_at
        updated_at = self.updated_at
        pushed_at = self.pushed_at
        git_url = self.git_url
        ssh_url = self.ssh_url
        clone_url = self.clone_url
        svn_url = self.svn_url
        homepage = self.homepage
        size = self.size
        stargazers_count = self.stargazers_count
        watchers_count = self.watchers_count
        language = self.language
        has_issues = self.has_issues
        has_projects = self.has_projects
        has_downloads = self.has_downloads
        has_wiki = self.has_wiki
        has_pages = self.has_pages
        forks_count = self.forks_count
        mirror_url = self.mirror_url
        archived = self.archived
        disabled = self.disabled
        open_issues_count = self.open_issues_count
        license_ = self.license_
        allow_forking = self.allow_forking
        is_template = self.is_template
        web_commit_signoff_required = self.web_commit_signoff_required
        topics: Union[Unset, List[str]] = UNSET
        if not isinstance(self.topics, Unset):
            topics = self.topics

        visibility = self.visibility
        default_branch = self.default_branch
        permissions: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.permissions, Unset):
            permissions = self.permissions.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if node_id is not UNSET:
            field_dict["node_id"] = node_id
        if name is not UNSET:
            field_dict["name"] = name
        if full_name is not UNSET:
            field_dict["full_name"] = full_name
        if private is not UNSET:
            field_dict["private"] = private
        if owner is not UNSET:
            field_dict["owner"] = owner
        if html_url is not UNSET:
            field_dict["html_url"] = html_url
        if description is not UNSET:
            field_dict["description"] = description
        if fork is not UNSET:
            field_dict["fork"] = fork
        if url is not UNSET:
            field_dict["url"] = url
        if forks_url is not UNSET:
            field_dict["forks_url"] = forks_url
        if keys_url is not UNSET:
            field_dict["keys_url"] = keys_url
        if collaborators_url is not UNSET:
            field_dict["collaborators_url"] = collaborators_url
        if teams_url is not UNSET:
            field_dict["teams_url"] = teams_url
        if hooks_url is not UNSET:
            field_dict["hooks_url"] = hooks_url
        if issue_events_url is not UNSET:
            field_dict["issue_events_url"] = issue_events_url
        if events_url is not UNSET:
            field_dict["events_url"] = events_url
        if assignees_url is not UNSET:
            field_dict["assignees_url"] = assignees_url
        if branches_url is not UNSET:
            field_dict["branches_url"] = branches_url
        if tags_url is not UNSET:
            field_dict["tags_url"] = tags_url
        if blobs_url is not UNSET:
            field_dict["blobs_url"] = blobs_url
        if git_tags_url is not UNSET:
            field_dict["git_tags_url"] = git_tags_url
        if git_refs_url is not UNSET:
            field_dict["git_refs_url"] = git_refs_url
        if trees_url is not UNSET:
            field_dict["trees_url"] = trees_url
        if statuses_url is not UNSET:
            field_dict["statuses_url"] = statuses_url
        if languages_url is not UNSET:
            field_dict["languages_url"] = languages_url
        if stargazers_url is not UNSET:
            field_dict["stargazers_url"] = stargazers_url
        if contributors_url is not UNSET:
            field_dict["contributors_url"] = contributors_url
        if subscribers_url is not UNSET:
            field_dict["subscribers_url"] = subscribers_url
        if subscription_url is not UNSET:
            field_dict["subscription_url"] = subscription_url
        if commits_url is not UNSET:
            field_dict["commits_url"] = commits_url
        if git_commits_url is not UNSET:
            field_dict["git_commits_url"] = git_commits_url
        if comments_url is not UNSET:
            field_dict["comments_url"] = comments_url
        if issue_comment_url is not UNSET:
            field_dict["issue_comment_url"] = issue_comment_url
        if contents_url is not UNSET:
            field_dict["contents_url"] = contents_url
        if compare_url is not UNSET:
            field_dict["compare_url"] = compare_url
        if merges_url is not UNSET:
            field_dict["merges_url"] = merges_url
        if archive_url is not UNSET:
            field_dict["archive_url"] = archive_url
        if downloads_url is not UNSET:
            field_dict["downloads_url"] = downloads_url
        if issues_url is not UNSET:
            field_dict["issues_url"] = issues_url
        if pulls_url is not UNSET:
            field_dict["pulls_url"] = pulls_url
        if milestones_url is not UNSET:
            field_dict["milestones_url"] = milestones_url
        if notifications_url is not UNSET:
            field_dict["notifications_url"] = notifications_url
        if labels_url is not UNSET:
            field_dict["labels_url"] = labels_url
        if releases_url is not UNSET:
            field_dict["releases_url"] = releases_url
        if deployments_url is not UNSET:
            field_dict["deployments_url"] = deployments_url
        if created_at is not UNSET:
            field_dict["created_at"] = created_at
        if updated_at is not UNSET:
            field_dict["updated_at"] = updated_at
        if pushed_at is not UNSET:
            field_dict["pushed_at"] = pushed_at
        if git_url is not UNSET:
            field_dict["git_url"] = git_url
        if ssh_url is not UNSET:
            field_dict["ssh_url"] = ssh_url
        if clone_url is not UNSET:
            field_dict["clone_url"] = clone_url
        if svn_url is not UNSET:
            field_dict["svn_url"] = svn_url
        if homepage is not UNSET:
            field_dict["homepage"] = homepage
        if size is not UNSET:
            field_dict["size"] = size
        if stargazers_count is not UNSET:
            field_dict["stargazers_count"] = stargazers_count
        if watchers_count is not UNSET:
            field_dict["watchers_count"] = watchers_count
        if language is not UNSET:
            field_dict["language"] = language
        if has_issues is not UNSET:
            field_dict["has_issues"] = has_issues
        if has_projects is not UNSET:
            field_dict["has_projects"] = has_projects
        if has_downloads is not UNSET:
            field_dict["has_downloads"] = has_downloads
        if has_wiki is not UNSET:
            field_dict["has_wiki"] = has_wiki
        if has_pages is not UNSET:
            field_dict["has_pages"] = has_pages
        if forks_count is not UNSET:
            field_dict["forks_count"] = forks_count
        if mirror_url is not UNSET:
            field_dict["mirror_url"] = mirror_url
        if archived is not UNSET:
            field_dict["archived"] = archived
        if disabled is not UNSET:
            field_dict["disabled"] = disabled
        if open_issues_count is not UNSET:
            field_dict["open_issues_count"] = open_issues_count
        if license_ is not UNSET:
            field_dict["license"] = license_
        if allow_forking is not UNSET:
            field_dict["allow_forking"] = allow_forking
        if is_template is not UNSET:
            field_dict["is_template"] = is_template
        if web_commit_signoff_required is not UNSET:
            field_dict["web_commit_signoff_required"] = web_commit_signoff_required
        if topics is not UNSET:
            field_dict["topics"] = topics
        if visibility is not UNSET:
            field_dict["visibility"] = visibility
        if default_branch is not UNSET:
            field_dict["default_branch"] = default_branch
        if permissions is not UNSET:
            field_dict["permissions"] = permissions

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.github_repo_owner import GithubRepoOwner
        from ..models.github_repo_permissions import GithubRepoPermissions

        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        node_id = d.pop("node_id", UNSET) or UNSET

        name = d.pop("name", UNSET) or UNSET

        full_name = d.pop("full_name", UNSET) or UNSET

        private = d.pop("private", UNSET) or UNSET

        _owner = d.pop("owner", UNSET) or UNSET
        owner: Union[Unset, GithubRepoOwner]
        if isinstance(_owner, Unset):
            owner = UNSET
        else:
            owner = GithubRepoOwner.from_dict(_owner)

        html_url = d.pop("html_url", UNSET) or UNSET

        description = d.pop("description", UNSET) or UNSET

        fork = d.pop("fork", UNSET) or UNSET

        url = d.pop("url", UNSET) or UNSET

        forks_url = d.pop("forks_url", UNSET) or UNSET

        keys_url = d.pop("keys_url", UNSET) or UNSET

        collaborators_url = d.pop("collaborators_url", UNSET) or UNSET

        teams_url = d.pop("teams_url", UNSET) or UNSET

        hooks_url = d.pop("hooks_url", UNSET) or UNSET

        issue_events_url = d.pop("issue_events_url", UNSET) or UNSET

        events_url = d.pop("events_url", UNSET) or UNSET

        assignees_url = d.pop("assignees_url", UNSET) or UNSET

        branches_url = d.pop("branches_url", UNSET) or UNSET

        tags_url = d.pop("tags_url", UNSET) or UNSET

        blobs_url = d.pop("blobs_url", UNSET) or UNSET

        git_tags_url = d.pop("git_tags_url", UNSET) or UNSET

        git_refs_url = d.pop("git_refs_url", UNSET) or UNSET

        trees_url = d.pop("trees_url", UNSET) or UNSET

        statuses_url = d.pop("statuses_url", UNSET) or UNSET

        languages_url = d.pop("languages_url", UNSET) or UNSET

        stargazers_url = d.pop("stargazers_url", UNSET) or UNSET

        contributors_url = d.pop("contributors_url", UNSET) or UNSET

        subscribers_url = d.pop("subscribers_url", UNSET) or UNSET

        subscription_url = d.pop("subscription_url", UNSET) or UNSET

        commits_url = d.pop("commits_url", UNSET) or UNSET

        git_commits_url = d.pop("git_commits_url", UNSET) or UNSET

        comments_url = d.pop("comments_url", UNSET) or UNSET

        issue_comment_url = d.pop("issue_comment_url", UNSET) or UNSET

        contents_url = d.pop("contents_url", UNSET) or UNSET

        compare_url = d.pop("compare_url", UNSET) or UNSET

        merges_url = d.pop("merges_url", UNSET) or UNSET

        archive_url = d.pop("archive_url", UNSET) or UNSET

        downloads_url = d.pop("downloads_url", UNSET) or UNSET

        issues_url = d.pop("issues_url", UNSET) or UNSET

        pulls_url = d.pop("pulls_url", UNSET) or UNSET

        milestones_url = d.pop("milestones_url", UNSET) or UNSET

        notifications_url = d.pop("notifications_url", UNSET) or UNSET

        labels_url = d.pop("labels_url", UNSET) or UNSET

        releases_url = d.pop("releases_url", UNSET) or UNSET

        deployments_url = d.pop("deployments_url", UNSET) or UNSET

        created_at = d.pop("created_at", UNSET) or UNSET

        updated_at = d.pop("updated_at", UNSET) or UNSET

        pushed_at = d.pop("pushed_at", UNSET) or UNSET

        git_url = d.pop("git_url", UNSET) or UNSET

        ssh_url = d.pop("ssh_url", UNSET) or UNSET

        clone_url = d.pop("clone_url", UNSET) or UNSET

        svn_url = d.pop("svn_url", UNSET) or UNSET

        homepage = d.pop("homepage", UNSET) or UNSET

        size = d.pop("size", UNSET) or UNSET

        stargazers_count = d.pop("stargazers_count", UNSET) or UNSET

        watchers_count = d.pop("watchers_count", UNSET) or UNSET

        language = d.pop("language", UNSET) or UNSET

        has_issues = d.pop("has_issues", UNSET) or UNSET

        has_projects = d.pop("has_projects", UNSET) or UNSET

        has_downloads = d.pop("has_downloads", UNSET) or UNSET

        has_wiki = d.pop("has_wiki", UNSET) or UNSET

        has_pages = d.pop("has_pages", UNSET) or UNSET

        forks_count = d.pop("forks_count", UNSET) or UNSET

        mirror_url = d.pop("mirror_url", UNSET) or UNSET

        archived = d.pop("archived", UNSET) or UNSET

        disabled = d.pop("disabled", UNSET) or UNSET

        open_issues_count = d.pop("open_issues_count", UNSET) or UNSET

        license_ = d.pop("license", UNSET) or UNSET

        allow_forking = d.pop("allow_forking", UNSET) or UNSET

        is_template = d.pop("is_template", UNSET) or UNSET

        web_commit_signoff_required = d.pop("web_commit_signoff_required", UNSET) or UNSET

        topics = cast(List[str], d.pop("topics", UNSET) or UNSET)

        visibility = d.pop("visibility", UNSET) or UNSET

        default_branch = d.pop("default_branch", UNSET) or UNSET

        _permissions = d.pop("permissions", UNSET) or UNSET
        permissions: Union[Unset, GithubRepoPermissions]
        if isinstance(_permissions, Unset):
            permissions = UNSET
        else:
            permissions = GithubRepoPermissions.from_dict(_permissions)

        github_repo = cls(
            id=id,
            node_id=node_id,
            name=name,
            full_name=full_name,
            private=private,
            owner=owner,
            html_url=html_url,
            description=description,
            fork=fork,
            url=url,
            forks_url=forks_url,
            keys_url=keys_url,
            collaborators_url=collaborators_url,
            teams_url=teams_url,
            hooks_url=hooks_url,
            issue_events_url=issue_events_url,
            events_url=events_url,
            assignees_url=assignees_url,
            branches_url=branches_url,
            tags_url=tags_url,
            blobs_url=blobs_url,
            git_tags_url=git_tags_url,
            git_refs_url=git_refs_url,
            trees_url=trees_url,
            statuses_url=statuses_url,
            languages_url=languages_url,
            stargazers_url=stargazers_url,
            contributors_url=contributors_url,
            subscribers_url=subscribers_url,
            subscription_url=subscription_url,
            commits_url=commits_url,
            git_commits_url=git_commits_url,
            comments_url=comments_url,
            issue_comment_url=issue_comment_url,
            contents_url=contents_url,
            compare_url=compare_url,
            merges_url=merges_url,
            archive_url=archive_url,
            downloads_url=downloads_url,
            issues_url=issues_url,
            pulls_url=pulls_url,
            milestones_url=milestones_url,
            notifications_url=notifications_url,
            labels_url=labels_url,
            releases_url=releases_url,
            deployments_url=deployments_url,
            created_at=created_at,
            updated_at=updated_at,
            pushed_at=pushed_at,
            git_url=git_url,
            ssh_url=ssh_url,
            clone_url=clone_url,
            svn_url=svn_url,
            homepage=homepage,
            size=size,
            stargazers_count=stargazers_count,
            watchers_count=watchers_count,
            language=language,
            has_issues=has_issues,
            has_projects=has_projects,
            has_downloads=has_downloads,
            has_wiki=has_wiki,
            has_pages=has_pages,
            forks_count=forks_count,
            mirror_url=mirror_url,
            archived=archived,
            disabled=disabled,
            open_issues_count=open_issues_count,
            license_=license_,
            allow_forking=allow_forking,
            is_template=is_template,
            web_commit_signoff_required=web_commit_signoff_required,
            topics=topics,
            visibility=visibility,
            default_branch=default_branch,
            permissions=permissions,
        )

        github_repo.additional_properties = d
        return github_repo

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.create_connection_request_dto_source import CreateConnectionRequestDtoSource


T = TypeVar("T", bound="CreateConnectionRequestDto")


@attr.s(auto_attribs=True)
class CreateConnectionRequestDto:
    """
    Attributes:
        user_id (str):
        client_id (str):
        source (CreateConnectionRequestDtoSource):
        redirect (str):
        source_policy_id (Union[Unset, str]):
        login (Union[Unset, bool]):
    """

    user_id: str
    client_id: str
    source: "CreateConnectionRequestDtoSource"
    redirect: str
    source_policy_id: Union[Unset, str] = UNSET
    login: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        user_id = self.user_id
        client_id = self.client_id
        source = self.source.to_dict()

        redirect = self.redirect
        source_policy_id = self.source_policy_id
        login = self.login

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "userId": user_id,
                "clientId": client_id,
                "source": source,
                "redirect": redirect,
            }
        )
        if source_policy_id is not UNSET:
            field_dict["sourcePolicyId"] = source_policy_id
        if login is not UNSET:
            field_dict["login"] = login

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.create_connection_request_dto_source import CreateConnectionRequestDtoSource

        d = src_dict.copy()
        user_id = d.pop("userId")

        client_id = d.pop("clientId")

        source = CreateConnectionRequestDtoSource.from_dict(d.pop("source"))

        redirect = d.pop("redirect")

        source_policy_id = d.pop("sourcePolicyId", UNSET) or UNSET

        login = d.pop("login", UNSET) or UNSET

        create_connection_request_dto = cls(
            user_id=user_id,
            client_id=client_id,
            source=source,
            redirect=redirect,
            source_policy_id=source_policy_id,
            login=login,
        )

        create_connection_request_dto.additional_properties = d
        return create_connection_request_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleEventsEventsItemConferenceDataEntryPointsItem")


@attr.s(auto_attribs=True)
class GoogleEventsEventsItemConferenceDataEntryPointsItem:
    """An entryPoint

    Attributes:
        entry_point_type (Union[Unset, str]): The entryPointType of the entryPoint
        uri (Union[Unset, str]): The uri of the entryPoint
        label (Union[Unset, str]): The label of the entryPoint
        pin (Union[Unset, str]): The pin of the entryPoint
        access_code (Union[Unset, str]): The accessCode of the entryPoint
        meeting_code (Union[Unset, str]): The meetingCode of the entryPoint
        passcode (Union[Unset, str]): The passcode of the entryPoint
        password (Union[Unset, str]): The password of the entryPoint
    """

    entry_point_type: Union[Unset, str] = UNSET
    uri: Union[Unset, str] = UNSET
    label: Union[Unset, str] = UNSET
    pin: Union[Unset, str] = UNSET
    access_code: Union[Unset, str] = UNSET
    meeting_code: Union[Unset, str] = UNSET
    passcode: Union[Unset, str] = UNSET
    password: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        entry_point_type = self.entry_point_type
        uri = self.uri
        label = self.label
        pin = self.pin
        access_code = self.access_code
        meeting_code = self.meeting_code
        passcode = self.passcode
        password = self.password

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if entry_point_type is not UNSET:
            field_dict["entryPointType"] = entry_point_type
        if uri is not UNSET:
            field_dict["uri"] = uri
        if label is not UNSET:
            field_dict["label"] = label
        if pin is not UNSET:
            field_dict["pin"] = pin
        if access_code is not UNSET:
            field_dict["accessCode"] = access_code
        if meeting_code is not UNSET:
            field_dict["meetingCode"] = meeting_code
        if passcode is not UNSET:
            field_dict["passcode"] = passcode
        if password is not UNSET:
            field_dict["password"] = password

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        entry_point_type = d.pop("entryPointType", UNSET) or UNSET

        uri = d.pop("uri", UNSET) or UNSET

        label = d.pop("label", UNSET) or UNSET

        pin = d.pop("pin", UNSET) or UNSET

        access_code = d.pop("accessCode", UNSET) or UNSET

        meeting_code = d.pop("meetingCode", UNSET) or UNSET

        passcode = d.pop("passcode", UNSET) or UNSET

        password = d.pop("password", UNSET) or UNSET

        google_events_events_item_conference_data_entry_points_item = cls(
            entry_point_type=entry_point_type,
            uri=uri,
            label=label,
            pin=pin,
            access_code=access_code,
            meeting_code=meeting_code,
            passcode=passcode,
            password=password,
        )

        google_events_events_item_conference_data_entry_points_item.additional_properties = d
        return google_events_events_item_conference_data_entry_points_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleEventsEventsItemAttendeesItem")


@attr.s(auto_attribs=True)
class GoogleEventsEventsItemAttendeesItem:
    """An attendee

    Attributes:
        id (Union[Unset, str]): The id of the attendee
        email (Union[Unset, str]): The email of the attendee
        display_name (Union[Unset, str]): The displayName of the attendee
        organizer (Union[Unset, bool]): Whether the attendee is the organizer
        self_ (Union[Unset, bool]): Whether the attendee is self
        resource (Union[Unset, bool]): Whether the attendee is a resource
        optional (Union[Unset, bool]): Whether the attendee is optional
        response_status (Union[Unset, str]): The responseStatus of the attendee
        comment (Union[Unset, str]): The comment of the attendee
        additional_guests (Union[Unset, float]): The additionalGuests of the attendee
    """

    id: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    display_name: Union[Unset, str] = UNSET
    organizer: Union[Unset, bool] = UNSET
    self_: Union[Unset, bool] = UNSET
    resource: Union[Unset, bool] = UNSET
    optional: Union[Unset, bool] = UNSET
    response_status: Union[Unset, str] = UNSET
    comment: Union[Unset, str] = UNSET
    additional_guests: Union[Unset, float] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        email = self.email
        display_name = self.display_name
        organizer = self.organizer
        self_ = self.self_
        resource = self.resource
        optional = self.optional
        response_status = self.response_status
        comment = self.comment
        additional_guests = self.additional_guests

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if email is not UNSET:
            field_dict["email"] = email
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if organizer is not UNSET:
            field_dict["organizer"] = organizer
        if self_ is not UNSET:
            field_dict["self"] = self_
        if resource is not UNSET:
            field_dict["resource"] = resource
        if optional is not UNSET:
            field_dict["optional"] = optional
        if response_status is not UNSET:
            field_dict["responseStatus"] = response_status
        if comment is not UNSET:
            field_dict["comment"] = comment
        if additional_guests is not UNSET:
            field_dict["additionalGuests"] = additional_guests

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        email = d.pop("email", UNSET) or UNSET

        display_name = d.pop("displayName", UNSET) or UNSET

        organizer = d.pop("organizer", UNSET) or UNSET

        self_ = d.pop("self", UNSET) or UNSET

        resource = d.pop("resource", UNSET) or UNSET

        optional = d.pop("optional", UNSET) or UNSET

        response_status = d.pop("responseStatus", UNSET) or UNSET

        comment = d.pop("comment", UNSET) or UNSET

        additional_guests = d.pop("additionalGuests", UNSET) or UNSET

        google_events_events_item_attendees_item = cls(
            id=id,
            email=email,
            display_name=display_name,
            organizer=organizer,
            self_=self_,
            resource=resource,
            optional=optional,
            response_status=response_status,
            comment=comment,
            additional_guests=additional_guests,
        )

        google_events_events_item_attendees_item.additional_properties = d
        return google_events_events_item_attendees_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_parsed_threads_threads_item_messages_item_headers_bcc_item import (
        GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem,
    )
    from ..models.google_parsed_threads_threads_item_messages_item_headers_cc_item import (
        GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem,
    )
    from ..models.google_parsed_threads_threads_item_messages_item_headers_from import (
        GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom,
    )
    from ..models.google_parsed_threads_threads_item_messages_item_headers_to_item import (
        GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem,
    )


T = TypeVar("T", bound="GoogleParsedThreadsThreadsItemMessagesItemHeaders")


@attr.s(auto_attribs=True)
class GoogleParsedThreadsThreadsItemMessagesItemHeaders:
    """The headers of the message

    Attributes:
        date (Union[Unset, str]): The date of the message
        subject (Union[Unset, str]): The subject of the message
        from_ (Union[Unset, GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom]): The writer of the message
        to (Union[Unset, List['GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem']]): The receivers of the message
        cc (Union[Unset, List['GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem']]): The ccs of the message
        bcc (Union[Unset, List['GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem']]): The bccs of the message
    """

    date: Union[Unset, str] = UNSET
    subject: Union[Unset, str] = UNSET
    from_: Union[Unset, "GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom"] = UNSET
    to: Union[Unset, List["GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem"]] = UNSET
    cc: Union[Unset, List["GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem"]] = UNSET
    bcc: Union[Unset, List["GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem"]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        date = self.date
        subject = self.subject
        from_: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.from_, Unset):
            from_ = self.from_.to_dict()

        to: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.to, Unset):
            to = []
            for to_item_data in self.to:
                to_item = to_item_data.to_dict()

                to.append(to_item)

        cc: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.cc, Unset):
            cc = []
            for cc_item_data in self.cc:
                cc_item = cc_item_data.to_dict()

                cc.append(cc_item)

        bcc: Union[Unset, List[Dict[str, Any]]] = UNSET
        if not isinstance(self.bcc, Unset):
            bcc = []
            for bcc_item_data in self.bcc:
                bcc_item = bcc_item_data.to_dict()

                bcc.append(bcc_item)

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if date is not UNSET:
            field_dict["date"] = date
        if subject is not UNSET:
            field_dict["subject"] = subject
        if from_ is not UNSET:
            field_dict["from"] = from_
        if to is not UNSET:
            field_dict["to"] = to
        if cc is not UNSET:
            field_dict["cc"] = cc
        if bcc is not UNSET:
            field_dict["bcc"] = bcc

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_parsed_threads_threads_item_messages_item_headers_bcc_item import (
            GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem,
        )
        from ..models.google_parsed_threads_threads_item_messages_item_headers_cc_item import (
            GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem,
        )
        from ..models.google_parsed_threads_threads_item_messages_item_headers_from import (
            GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom,
        )
        from ..models.google_parsed_threads_threads_item_messages_item_headers_to_item import (
            GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem,
        )

        d = src_dict.copy()
        date = d.pop("date", UNSET) or UNSET

        subject = d.pop("subject", UNSET) or UNSET

        _from_ = d.pop("from", UNSET) or UNSET
        from_: Union[Unset, GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom]
        if isinstance(_from_, Unset):
            from_ = UNSET
        else:
            from_ = GoogleParsedThreadsThreadsItemMessagesItemHeadersFrom.from_dict(_from_)

        to = []
        _to = d.pop("to", UNSET) or UNSET
        for to_item_data in _to or []:
            to_item = GoogleParsedThreadsThreadsItemMessagesItemHeadersToItem.from_dict(to_item_data)

            to.append(to_item)

        cc = []
        _cc = d.pop("cc", UNSET) or UNSET
        for cc_item_data in _cc or []:
            cc_item = GoogleParsedThreadsThreadsItemMessagesItemHeadersCcItem.from_dict(cc_item_data)

            cc.append(cc_item)

        bcc = []
        _bcc = d.pop("bcc", UNSET) or UNSET
        for bcc_item_data in _bcc or []:
            bcc_item = GoogleParsedThreadsThreadsItemMessagesItemHeadersBccItem.from_dict(bcc_item_data)

            bcc.append(bcc_item)

        google_parsed_threads_threads_item_messages_item_headers = cls(
            date=date,
            subject=subject,
            from_=from_,
            to=to,
            cc=cc,
            bcc=bcc,
        )

        google_parsed_threads_threads_item_messages_item_headers.additional_properties = d
        return google_parsed_threads_threads_item_messages_item_headers

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleEventCreator")


@attr.s(auto_attribs=True)
class GoogleEventCreator:
    """The creator of the event

    Attributes:
        id (Union[Unset, str]): The id of the creator
        email (Union[Unset, str]): The email of the creator
        display_name (Union[Unset, str]): The displayName of the creator
        self_ (Union[Unset, bool]): Whether the creator is self
    """

    id: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    display_name: Union[Unset, str] = UNSET
    self_: Union[Unset, bool] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        email = self.email
        display_name = self.display_name
        self_ = self.self_

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if id is not UNSET:
            field_dict["id"] = id
        if email is not UNSET:
            field_dict["email"] = email
        if display_name is not UNSET:
            field_dict["displayName"] = display_name
        if self_ is not UNSET:
            field_dict["self"] = self_

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id", UNSET) or UNSET

        email = d.pop("email", UNSET) or UNSET

        display_name = d.pop("displayName", UNSET) or UNSET

        self_ = d.pop("self", UNSET) or UNSET

        google_event_creator = cls(
            id=id,
            email=email,
            display_name=display_name,
            self_=self_,
        )

        google_event_creator.additional_properties = d
        return google_event_creator

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.google_events_events_item_gadget_preferences import GoogleEventsEventsItemGadgetPreferences


T = TypeVar("T", bound="GoogleEventsEventsItemGadget")


@attr.s(auto_attribs=True)
class GoogleEventsEventsItemGadget:
    """The gadget of the event

    Attributes:
        type (Union[Unset, str]): The type of the gadget
        title (Union[Unset, str]): The title of the gadget
        link (Union[Unset, str]): The link of the gadget
        icon_link (Union[Unset, str]): The iconLink of the gadget
        width (Union[Unset, float]): The width of the gadget
        height (Union[Unset, float]): The height of the gadget
        display (Union[Unset, str]): The display of the gadget
        preferences (Union[Unset, GoogleEventsEventsItemGadgetPreferences]): The preferences of the gadget
    """

    type: Union[Unset, str] = UNSET
    title: Union[Unset, str] = UNSET
    link: Union[Unset, str] = UNSET
    icon_link: Union[Unset, str] = UNSET
    width: Union[Unset, float] = UNSET
    height: Union[Unset, float] = UNSET
    display: Union[Unset, str] = UNSET
    preferences: Union[Unset, "GoogleEventsEventsItemGadgetPreferences"] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        type = self.type
        title = self.title
        link = self.link
        icon_link = self.icon_link
        width = self.width
        height = self.height
        display = self.display
        preferences: Union[Unset, Dict[str, Any]] = UNSET
        if not isinstance(self.preferences, Unset):
            preferences = self.preferences.to_dict()

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if type is not UNSET:
            field_dict["type"] = type
        if title is not UNSET:
            field_dict["title"] = title
        if link is not UNSET:
            field_dict["link"] = link
        if icon_link is not UNSET:
            field_dict["iconLink"] = icon_link
        if width is not UNSET:
            field_dict["width"] = width
        if height is not UNSET:
            field_dict["height"] = height
        if display is not UNSET:
            field_dict["display"] = display
        if preferences is not UNSET:
            field_dict["preferences"] = preferences

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.google_events_events_item_gadget_preferences import GoogleEventsEventsItemGadgetPreferences

        d = src_dict.copy()
        type = d.pop("type", UNSET) or UNSET

        title = d.pop("title", UNSET) or UNSET

        link = d.pop("link", UNSET) or UNSET

        icon_link = d.pop("iconLink", UNSET) or UNSET

        width = d.pop("width", UNSET) or UNSET

        height = d.pop("height", UNSET) or UNSET

        display = d.pop("display", UNSET) or UNSET

        _preferences = d.pop("preferences", UNSET) or UNSET
        preferences: Union[Unset, GoogleEventsEventsItemGadgetPreferences]
        if isinstance(_preferences, Unset):
            preferences = UNSET
        else:
            preferences = GoogleEventsEventsItemGadgetPreferences.from_dict(_preferences)

        google_events_events_item_gadget = cls(
            type=type,
            title=title,
            link=link,
            icon_link=icon_link,
            width=width,
            height=height,
            display=display,
            preferences=preferences,
        )

        google_events_events_item_gadget.additional_properties = d
        return google_events_events_item_gadget

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleCalendarsCalendarsItemConferenceProperties")


@attr.s(auto_attribs=True)
class GoogleCalendarsCalendarsItemConferenceProperties:
    """The conferenceProperties of the calendar

    Attributes:
        allowed_conference_solution_types (Union[Unset, List[str]]): The allowedConferenceSolutionTypes of the
            conferenceProperties
    """

    allowed_conference_solution_types: Union[Unset, List[str]] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        allowed_conference_solution_types: Union[Unset, List[str]] = UNSET
        if not isinstance(self.allowed_conference_solution_types, Unset):
            allowed_conference_solution_types = self.allowed_conference_solution_types

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if allowed_conference_solution_types is not UNSET:
            field_dict["allowedConferenceSolutionTypes"] = allowed_conference_solution_types

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        allowed_conference_solution_types = cast(List[str], d.pop("allowedConferenceSolutionTypes", UNSET) or UNSET)

        google_calendars_calendars_item_conference_properties = cls(
            allowed_conference_solution_types=allowed_conference_solution_types,
        )

        google_calendars_calendars_item_conference_properties.additional_properties = d
        return google_calendars_calendars_item_conference_properties

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union, cast

import attr

from ..models.credential_provider import CredentialProvider
from ..models.source_policy_status import SourcePolicyStatus
from ..models.source_type import SourceType
from ..types import UNSET, Unset

T = TypeVar("T", bound="ReturnedSourcePolicyDto")


@attr.s(auto_attribs=True)
class ReturnedSourcePolicyDto:
    """
    Attributes:
        id (str):
        source (SourceType):
        status (SourcePolicyStatus):
        default (bool):
        access_credential_provider (CredentialProvider):
        allowed (List[str]):
        allow_all (bool):
        client_id (str):
        name (Union[Unset, str]):
        auth_credential_provider (Union[Unset, CredentialProvider]):
    """

    id: str
    source: SourceType
    status: SourcePolicyStatus
    default: bool
    access_credential_provider: CredentialProvider
    allowed: List[str]
    allow_all: bool
    client_id: str
    name: Union[Unset, str] = UNSET
    auth_credential_provider: Union[Unset, CredentialProvider] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        source = self.source.value

        status = self.status.value

        default = self.default
        access_credential_provider = self.access_credential_provider.value

        allowed = self.allowed

        allow_all = self.allow_all
        client_id = self.client_id
        name = self.name
        auth_credential_provider: Union[Unset, str] = UNSET
        if not isinstance(self.auth_credential_provider, Unset):
            auth_credential_provider = self.auth_credential_provider.value

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "source": source,
                "status": status,
                "default": default,
                "accessCredentialProvider": access_credential_provider,
                "allowed": allowed,
                "allowAll": allow_all,
                "clientId": client_id,
            }
        )
        if name is not UNSET:
            field_dict["name"] = name
        if auth_credential_provider is not UNSET:
            field_dict["authCredentialProvider"] = auth_credential_provider

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        id = d.pop("id")

        source = SourceType(d.pop("source"))

        status = SourcePolicyStatus(d.pop("status"))

        default = d.pop("default")

        access_credential_provider = CredentialProvider(d.pop("accessCredentialProvider"))

        allowed = cast(List[str], d.pop("allowed"))

        allow_all = d.pop("allowAll")

        client_id = d.pop("clientId")

        name = d.pop("name", UNSET) or UNSET

        _auth_credential_provider = d.pop("authCredentialProvider", UNSET) or UNSET
        auth_credential_provider: Union[Unset, CredentialProvider]
        if isinstance(_auth_credential_provider, Unset):
            auth_credential_provider = UNSET
        else:
            auth_credential_provider = CredentialProvider(_auth_credential_provider)

        returned_source_policy_dto = cls(
            id=id,
            source=source,
            status=status,
            default=default,
            access_credential_provider=access_credential_provider,
            allowed=allowed,
            allow_all=allow_all,
            client_id=client_id,
            name=name,
            auth_credential_provider=auth_credential_provider,
        )

        returned_source_policy_dto.additional_properties = d
        return returned_source_policy_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import TYPE_CHECKING, Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

if TYPE_CHECKING:
    from ..models.returned_connection_request_dto_source import ReturnedConnectionRequestDtoSource


T = TypeVar("T", bound="ReturnedConnectionRequestDto")


@attr.s(auto_attribs=True)
class ReturnedConnectionRequestDto:
    """
    Attributes:
        id (str):
        source (ReturnedConnectionRequestDtoSource):
        status (str):
        client_id (str):
        user_id (str):
        source_policy_id (str):
        redirect (Union[Unset, str]):
        auth_url (Union[Unset, str]):
        account_id (Union[Unset, str]):
    """

    id: str
    source: "ReturnedConnectionRequestDtoSource"
    status: str
    client_id: str
    user_id: str
    source_policy_id: str
    redirect: Union[Unset, str] = UNSET
    auth_url: Union[Unset, str] = UNSET
    account_id: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        id = self.id
        source = self.source.to_dict()

        status = self.status
        client_id = self.client_id
        user_id = self.user_id
        source_policy_id = self.source_policy_id
        redirect = self.redirect
        auth_url = self.auth_url
        account_id = self.account_id

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update(
            {
                "id": id,
                "source": source,
                "status": status,
                "clientId": client_id,
                "userId": user_id,
                "sourcePolicyId": source_policy_id,
            }
        )
        if redirect is not UNSET:
            field_dict["redirect"] = redirect
        if auth_url is not UNSET:
            field_dict["authUrl"] = auth_url
        if account_id is not UNSET:
            field_dict["accountId"] = account_id

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        from ..models.returned_connection_request_dto_source import ReturnedConnectionRequestDtoSource

        d = src_dict.copy()
        id = d.pop("id")

        source = ReturnedConnectionRequestDtoSource.from_dict(d.pop("source"))

        status = d.pop("status")

        client_id = d.pop("clientId")

        user_id = d.pop("userId")

        source_policy_id = d.pop("sourcePolicyId")

        redirect = d.pop("redirect", UNSET) or UNSET

        auth_url = d.pop("authUrl", UNSET) or UNSET

        account_id = d.pop("accountId", UNSET) or UNSET

        returned_connection_request_dto = cls(
            id=id,
            source=source,
            status=status,
            client_id=client_id,
            user_id=user_id,
            source_policy_id=source_policy_id,
            redirect=redirect,
            auth_url=auth_url,
            account_id=account_id,
        )

        returned_connection_request_dto.additional_properties = d
        return returned_connection_request_dto

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties



from typing import Any, Dict, List, Type, TypeVar, Union

import attr

from ..types import UNSET, Unset

T = TypeVar("T", bound="GoogleParsedMessageHeadersBccItem")


@attr.s(auto_attribs=True)
class GoogleParsedMessageHeadersBccItem:
    """A bcc of the message

    Attributes:
        name (Union[Unset, str]): The name of the bcc
        email (Union[Unset, str]): The email of the bcc
    """

    name: Union[Unset, str] = UNSET
    email: Union[Unset, str] = UNSET
    additional_properties: Dict[str, Any] = attr.ib(init=False, factory=dict)

    def to_dict(self) -> Dict[str, Any]:
        name = self.name
        email = self.email

        field_dict: Dict[str, Any] = {}
        field_dict.update(self.additional_properties)
        field_dict.update({})
        if name is not UNSET:
            field_dict["name"] = name
        if email is not UNSET:
            field_dict["email"] = email

        return field_dict

    @classmethod
    def from_dict(cls: Type[T], src_dict: Dict[str, Any]) -> T:
        d = src_dict.copy()
        name = d.pop("name", UNSET) or UNSET

        email = d.pop("email", UNSET) or UNSET

        google_parsed_message_headers_bcc_item = cls(
            name=name,
            email=email,
        )

        google_parsed_message_headers_bcc_item.additional_properties = d
        return google_parsed_message_headers_bcc_item

    @property
    def additional_keys(self) -> List[str]:
        return list(self.additional_properties.keys())

    def __getitem__(self, key: str) -> Any:
        return self.additional_properties[key]

    def __setitem__(self, key: str, value: Any) -> None:
        self.additional_properties[key] = value

    def __delitem__(self, key: str) -> None:
        del self.additional_properties[key]

    def __contains__(self, key: str) -> bool:
        return key in self.additional_properties
